@@ -21,7 +21,7 @@
  * This demuxer detects ADIF and ADTS headers in AAC files.
  * Then it shovels buffer-sized chunks over to the AAC decoder.
  *
- * $Id: demux_aac.c,v 1.11 2006/03/11 09:21:25 tmattern Exp $
+ * $Id: demux_aac.c,v 1.13 2007/01/19 00:26:39 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -108,8 +108,12 @@ static int open_aac_file(demux_aac_t *this) {
 
     if ((frame_size > 0) &&
         (this->data_start+frame_size < MAX_PREVIEW_SIZE-1) &&
-        (peak[this->data_start+frame_size] == 0xff) &&
-        ((peak[this->data_start+frame_size+1] & 0xf6) == 0xf0)) {
+        /* first 28 bits must be identical */
+        (peak[this->data_start  ]   ==peak[this->data_start+frame_size  ]) &&
+        (peak[this->data_start+1]   ==peak[this->data_start+frame_size+1]) &&
+        (peak[this->data_start+2]   ==peak[this->data_start+frame_size+2]) &&
+        (peak[this->data_start+3]>>4==peak[this->data_start+frame_size+3]>>4))
+    {
       lprintf("found second ADTS header\n");
 
       _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 0);
@@ -251,7 +255,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   switch (stream->content_detection_method) {
 
   case METHOD_BY_EXTENSION: {
-    char *extensions, *mrl;
+    const char *extensions, *mrl;
 
     mrl = input->get_mrl (input);
     extensions = class_gen->get_extensions (class_gen);
@@ -279,19 +283,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "ADIF/ADTS AAC demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "AAC";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "aac";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
 
