@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: demux_avi.c,v 1.218 2005/02/06 15:26:00 tmattern Exp $
+ * $Id: demux_avi.c,v 1.229 2007/01/19 00:26:40 dgp85 Exp $
  *
  * demultiplexer for avi streams
  *
@@ -269,8 +269,7 @@ typedef struct demux_avi_s {
   off_t                seek_start_pos;
   int                  seek_start_time;
 
-  /* discontinuity detection */
-  int64_t              last_pts[2];
+  /* discontinuity detection (only at seek) */
   int                  send_newpts;
   int                  buf_flag_seek;
   
@@ -345,13 +344,10 @@ static uint32_t odml_key (unsigned char *str)
 }
 
 static void check_newpts (demux_avi_t *this, int64_t pts, int video) {
-  int64_t diff;
 
-  diff = pts - this->last_pts[video];
+  if (this->send_newpts) {
 
-  if (pts && (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD)) ) {
-
-    lprintf ("sending newpts %lld (video = %d diff = %lld)\n", pts, video, diff);
+    lprintf ("sending newpts %" PRId64 " (video = %d)\n", pts, video);
 
     if (this->buf_flag_seek) {
       _x_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
@@ -361,12 +357,7 @@ static void check_newpts (demux_avi_t *this, int64_t pts, int video) {
     }
 
     this->send_newpts = 0;
-    this->last_pts[1-video] = 0;
   }
-
-  if (pts)
-    this->last_pts[video] = pts;
-
 }
 
 /* Append an index entry for a newly-found video frame */
@@ -424,32 +415,32 @@ static int64_t get_audio_pts (demux_avi_t *this, int track, uint32_t posc,
 
   avi_audio_t *at = this->avi->audio[track];
 
-  lprintf("get_audio_pts: track=%d, posc=%d, postot=%lld, posb=%d\n", track, posc, postot, posb);
+  lprintf("get_audio_pts: track=%d, posc=%d, postot=%" PRIdMAX ", posb=%d\n", track, posc, (intmax_t)postot, posb);
 
   if ((at->dwSampleSize == 0) && (at->dwScale > 1)) {
     /* variable bitrate */
     lprintf("get_audio_pts: VBR: nBlockAlign=%d, dwSampleSize=%d, dwScale=%d, dwRate=%d\n",
             at->wavex->nBlockAlign, at->dwSampleSize, at->dwScale, at->dwRate);
-    return (int64_t)(90000.0 * (double)(posc) *
+    return (int64_t)(90000.0 * (double)(posc + at->dwStart) *
       (double)at->dwScale / (double)at->dwRate);
   } else {
     /* constant bitrate */
     lprintf("get_audio_pts: CBR: nBlockAlign=%d, dwSampleSize=%d, dwScale=%d, dwRate=%d\n",
             at->wavex->nBlockAlign, at->dwSampleSize, at->dwScale, at->dwRate);
     if( at->wavex && at->wavex->nBlockAlign ) {
-      return (int64_t)((double)(postot + posb) / (double)at->wavex->nBlockAlign *
+      return (int64_t)((double)((postot + posb) / (double)at->wavex->nBlockAlign + at->dwStart) *
         (double)at->dwScale / (double)at->dwRate * 90000.0);
     } else {
-      return (int64_t)((double)(postot + posb) / (double)at->dwSampleSize *
+      return (int64_t)((double)((postot + posb) / (double)at->dwSampleSize + at->dwStart) *
         (double)at->dwScale / (double)at->dwRate * 90000.0);
     }
   }
 }
 
 static int64_t get_video_pts (demux_avi_t *this, off_t pos) {
-  lprintf("get_video_pts: dwScale=%d, dwRate=%d, pos=%lld\n",
-         this->avi->dwScale, this->avi->dwRate, pos);
-  return (int64_t)(90000.0 * (double)pos *
+  lprintf("get_video_pts: dwScale=%d, dwRate=%d, pos=%" PRIdMAX "\n",
+         this->avi->dwScale, this->avi->dwRate, (intmax_t)pos);
+  return (int64_t)(90000.0 * (double)(pos + this->avi->dwStart) *
     (double)this->avi->dwScale / (double)this->avi->dwRate);
 }
 
@@ -477,7 +468,7 @@ static int audio_pos_stopper(demux_avi_t *this, void *data) {
  * is in the index. */
 static int start_pos_stopper(demux_avi_t *this, void *data) {
   off_t start_pos = *(off_t *)data;
-  uint32_t maxframe = this->avi->video_idx.video_frames - 1;
+  int32_t maxframe = this->avi->video_idx.video_frames - 1;
 
   while( maxframe >= 0 && this->avi->video_idx.vindex[maxframe].pos >= start_pos ) {
     if ( this->avi->video_idx.vindex[maxframe].flags & AVIIF_KEYFRAME )
@@ -491,7 +482,7 @@ static int start_pos_stopper(demux_avi_t *this, void *data) {
  * is in the index. */
 static int start_time_stopper(demux_avi_t *this, void *data) {
   int64_t video_pts = *(int64_t *)data;
-  uint32_t maxframe = this->avi->video_idx.video_frames - 1;
+  int32_t maxframe = this->avi->video_idx.video_frames - 1;
 
   while( maxframe >= 0 && get_video_pts(this,maxframe) >= video_pts ) {
     if ( this->avi->video_idx.vindex[maxframe].flags & AVIIF_KEYFRAME )
@@ -552,7 +543,7 @@ static int idx_grow(demux_avi_t *this, int (*stopper)(demux_avi_t *, void *),
     }
 
     if (this->input->read(this->input, data, AVI_HEADER_SIZE) != AVI_HEADER_SIZE) {
-      lprintf("read failed, chunk_pos=%lld\n", chunk_pos);
+      lprintf("read failed, chunk_pos=%" PRIdMAX "\n", (intmax_t)chunk_pos);
       break;
     }
 
@@ -643,7 +634,7 @@ static int idx_grow(demux_avi_t *this, int (*stopper)(demux_avi_t *, void *),
     }
     chunk_pos = this->input->seek(this->input, this->idx_grow.nexttagoffset, SEEK_SET);
     if (chunk_pos != this->idx_grow.nexttagoffset) {
-      lprintf("seek failed: %lld != %lld\n", chunk_pos, this->idx_grow.nexttagoffset);
+      lprintf("seek failed: %" PRIdMAX " != %" PRIdMAX "\n", (intmax_t)chunk_pos, (intmax_t)this->idx_grow.nexttagoffset);
       break;
     }
   }
@@ -791,15 +782,10 @@ static avi_t *AVI_init(demux_avi_t *this) {
     n = PAD_EVEN(n);
     next_chunk = this->idx_grow.nexttagoffset + 8 + n;
     
-    if (n == 0) {
-      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, "invalid chunk length (0 byte)\n");
-      break;
-    }
-    
-    lprintf("chunk: %c%c%c%c, size: %lld\n",
+    lprintf("chunk: %c%c%c%c, size: %" PRId64 "\n",
             data[0], data[1], data[2], data[3], (int64_t)n);
     
-    if(strncasecmp(data,"LIST",4) == 0) {
+    if((strncasecmp(data,"LIST",4) == 0) && (n >= 4)) {
       if( this->input->read(this->input, data,4) != 4 ) ERR_EXIT(AVI_ERR_READ);
       n -= 4;
       
@@ -1043,7 +1029,14 @@ static avi_t *AVI_init(demux_avi_t *this) {
          lprintf("Invalid Header, bIndexSubType != 0\n");
       }
 
-      superindex->aIndex = malloc (superindex->wLongsPerEntry * superindex->nEntriesInUse * sizeof (uint32_t));
+      if (superindex->nEntriesInUse > n / sizeof (avisuperindex_entry))
+      {
+         lprintf("broken index !, dwSize=%d, entries=%d\n", n, superindex->nEntriesInUse);
+         i += 8 + n;
+         continue;
+      }
+
+      superindex->aIndex = malloc (superindex->nEntriesInUse * sizeof (avisuperindex_entry));
       /* position of ix## chunks */
       for (j = 0; j < superindex->nEntriesInUse; ++j) {
         superindex->aIndex[j].qwOffset = LE_64 (a);   a += 8;
@@ -1399,7 +1392,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
   AVI->video_posf = 0;
   AVI->video_posb = 0;
 
-  lprintf("done, pos=%lld, AVI->movi_start=%lld\n", this->input->get_current_pos(this->input), AVI->movi_start);
+  lprintf("done, pos=%lld, AVI->movi_start=%" PRIdMAX "\n", this->input->get_current_pos(this->input), (intmax_t)AVI->movi_start);
   return AVI;
 }
 
@@ -1557,7 +1550,7 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
     audio_pts =
       get_audio_pts (this, i, aie->block_no, aie->tot, audio->audio_posb);
 
-    lprintf ("video_pts %lld audio_pts %lld\n", video_pts, audio_pts);
+    lprintf ("video_pts %" PRId64 " audio_pts %" PRId64 "\n", video_pts, audio_pts);
 
     if (!this->no_audio && (audio_pts < video_pts)) {
 
@@ -1657,9 +1650,9 @@ static int get_chunk_header(demux_avi_t *this, uint32_t *len, int *audio_stream)
       break;
     *len = LE_32(data + 4);
 
-    lprintf("header: %c%c%c%c, pos=%lld, len=%u\n",
+    lprintf("header: %c%c%c%c, pos=%" PRIdMAX ", len=%u\n",
             data[0], data[1], data[2], data[3],
-            this->input->get_current_pos(this->input), *len);
+            (intmax_t)this->input->get_current_pos(this->input), *len);
 
     /* Dive into RIFF and LIST entries */
     if(strncasecmp(data, "LIST", 4) == 0 ||
@@ -1708,13 +1701,13 @@ static int demux_avi_next_streaming (demux_avi_t *this, int decoder_flags) {
 
   buf_element_t *buf = NULL;
   int64_t        audio_pts, video_pts;
-  int64_t        current_pos;
+  off_t          current_pos;
   int            left;
   int            header, chunk_len, audio_stream;
   avi_audio_t   *audio;
 
   current_pos = this->input->get_current_pos(this->input);
-  lprintf("input_pos=%lld\n", current_pos);
+  lprintf("input_pos=%" PRIdMAX "\n", (intmax_t)current_pos);
 
   header = get_chunk_header(this, &chunk_len, &audio_stream);
 
@@ -1734,6 +1727,7 @@ static int demux_avi_next_streaming (demux_avi_t *this, int decoder_flags) {
 
         /* read audio */
         buf->pts = audio_pts;
+        lprintf("audio pts: %" PRId64 "\n", audio_pts);
 
         if (left > this->audio_fifo->buffer_pool_buf_size) {
           buf->size = this->audio_fifo->buffer_pool_buf_size;
@@ -1775,11 +1769,12 @@ static int demux_avi_next_streaming (demux_avi_t *this, int decoder_flags) {
       while (left > 0) {
         video_pts = get_video_pts (this, this->avi->video_posf);
 
-
         buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
 
         /* read video */
         buf->pts = video_pts;
+        lprintf("video pts: %" PRId64 "\n", video_pts);
+        
         if (left > this->video_fifo->buffer_pool_buf_size) {
           buf->size = this->video_fifo->buffer_pool_buf_size;
           buf->decoder_flags = 0;
@@ -1793,12 +1788,11 @@ static int demux_avi_next_streaming (demux_avi_t *this, int decoder_flags) {
           return 0;
         }
 
-        buf->type       = this->avi->video_type;
+        buf->type = this->avi->video_type;
         buf->extra_info->input_time = video_pts / 90;
         buf->extra_info->input_normpos = 65535;
         buf->extra_info->frame_number = this->avi->video_posf;
         buf->decoder_flags |= decoder_flags;
-
         this->video_fifo->put (this->video_fifo, buf);
       }
 
@@ -2050,12 +2044,14 @@ static void demux_avi_send_headers (demux_plugin_t *this_gen) {
 static int demux_avi_seek (demux_plugin_t *this_gen,
                            off_t start_pos, int start_time, int playing) {
   demux_avi_t *this = (demux_avi_t *) this_gen;
-  
+
   if (!this->streaming) {
+
     _x_demux_flush_engine (this->stream);
     this->seek_request    = 1;
     this->seek_start_pos  = start_pos;
     this->seek_start_time = start_time;
+
     this->status = DEMUX_OK;
   }
   return this->status;
@@ -2151,7 +2147,7 @@ static int demux_avi_seek_internal (demux_avi_t *this) {
    * position we've already found, so we won't be seeking though the
    * file much at this point. */
 
-  xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "video_pts = %lld\n", video_pts);
+  xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "video_pts = %" PRId64 "\n", video_pts);
 
   /* FIXME ? */
   audio_pts = 77777777;
@@ -2163,7 +2159,7 @@ static int demux_avi_seek_internal (demux_avi_t *this) {
     for(i = 0; i < this->avi->n_audio; i++) {
       max_pos = this->avi->audio[i]->audio_idx.audio_chunks - 1;
       min_pos = 0;
-      lprintf("audio_chunks=%d, min=%lld, max=%lld\n", this->avi->audio[i]->audio_idx.audio_chunks, min_pos, max_pos);
+      lprintf("audio_chunks=%d, min=%" PRId64 ", max=%" PRId64 "\n", this->avi->audio[i]->audio_idx.audio_chunks, min_pos, max_pos);
       while (min_pos < max_pos) {
         cur_pos = this->avi->audio[i]->audio_posc = (max_pos + min_pos) / 2;
         if (cur_pos == min_pos) break;
@@ -2174,7 +2170,7 @@ static int demux_avi_seek_internal (demux_avi_t *this) {
           } else {
             min_pos = cur_pos;
           }
-          lprintf ("audio_pts = %lld %lld < %lld < %lld\n",
+          lprintf ("audio_pts = %" PRId64 " %" PRId64 " < %" PRId64 " < %" PRId64 "\n",
                    audio_pts, min_pos, cur_pos, max_pos);
         } else {
           if (cur_pos > min_pos) {
@@ -2186,7 +2182,7 @@ static int demux_avi_seek_internal (demux_avi_t *this) {
           }
         }
       }
-      lprintf ("audio_pts = %lld\n", audio_pts);
+      lprintf ("audio_pts = %" PRId64 "\n", audio_pts);
 
       /*
        * try to make audio pos more accurate for long index entries
@@ -2263,7 +2259,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   break;
 
   case METHOD_BY_EXTENSION: {
-    char *extensions, *mrl;
+    const char *extensions, *mrl;
 
     mrl = input->get_mrl (input);
     extensions = class_gen->get_extensions (class_gen);
@@ -2320,19 +2316,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
  * demux avi class
  */
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "AVI/RIFF demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "AVI";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "avi";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return "video/msvideo: avi: AVI video;"
          "video/x-msvideo: avi: AVI video;";
 }
@@ -2361,11 +2357,11 @@ static void *init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_avi = {
+static const demuxer_info_t demux_info_avi = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "avi", XINE_VERSION_CODE, &demux_info_avi, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
