@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_mpeg_block.c,v 1.211 2005/02/06 15:26:16 tmattern Exp $
+ * $Id: demux_mpeg_block.c,v 1.219 2007/01/19 01:05:24 dgp85 Exp $
  *
  * demultiplexer for mpeg 1/2 program streams
  * used with fixed blocksize devices (like dvd/vcd)
@@ -72,7 +72,8 @@ typedef struct demux_mpeg_block_s {
 
   char                  cur_mrl[256];
 
-  uint8_t              *scratch, *scratch_base;
+  uint8_t              *scratch;
+  void                 *scratch_base;
 
   int64_t               nav_last_end_pts;
   int64_t               nav_last_start_pts;
@@ -1268,7 +1269,6 @@ static int demux_mpeg_block_seek (demux_plugin_t *this_gen,
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
   start_pos = (off_t) ( (double) start_pos / 65535 *
               this->input->get_length (this->input) );
-  start_time /= 1000;
 
   if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
     
@@ -1279,20 +1279,26 @@ static int demux_mpeg_block_seek (demux_plugin_t *this_gen,
       this->input->seek (this->input, start_pos, SEEK_SET);
     } else if (start_time) {
       
-      if (this->last_cell_time) {
-        start_pos = start_time - (this->last_cell_time + this->last_begin_time)/1000;
-        start_pos *= this->rate;
-        start_pos *= 50;
-        start_pos += this->last_cell_pos;
+      if( this->input->seek_time ) {
+        this->input->seek_time (this->input, start_time, SEEK_SET);
       } else {
-        start_pos = start_time;
-        start_pos *= this->rate;
-        start_pos *= 50;
+        start_time /= 1000;
+  
+        if (this->last_cell_time) {
+          start_pos = start_time - (this->last_cell_time + this->last_begin_time)/1000;
+          start_pos *= this->rate;
+          start_pos *= 50;
+          start_pos += this->last_cell_pos;
+        } else {
+          start_pos = start_time;
+          start_pos *= this->rate;
+          start_pos *= 50;
+        }
+        start_pos /= (off_t) this->blocksize;
+        start_pos *= (off_t) this->blocksize;
+        
+        this->input->seek (this->input, start_pos, SEEK_SET);
       }
-      start_pos /= (off_t) this->blocksize;
-      start_pos *= (off_t) this->blocksize;
-      
-      this->input->seek (this->input, start_pos, SEEK_SET);
     } else
       this->input->seek (this->input, 0, SEEK_SET);
   }
@@ -1382,7 +1388,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.get_optional_data = demux_mpeg_block_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
 
-  this->scratch    = xine_xmalloc_aligned (512, 4096, (void**) &this->scratch_base);
+  this->scratch    = xine_xmalloc_aligned (512, 4096, &this->scratch_base);
   this->status     = DEMUX_FINISHED;
 
   lprintf ("open_plugin:detection_method=%d\n",
@@ -1450,9 +1456,9 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   break;
 
   case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
+    char *ending;
 
-    mrl = input->get_mrl (input);
+    const char *const mrl = input->get_mrl (input);
 
     if(!strncmp(mrl, "vcd:", 4)) {
       this->blocksize = 2324;
@@ -1510,19 +1516,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "DVD/VOB demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "MPEG_BLOCK";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "vob";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
 
@@ -1554,11 +1560,11 @@ static void *init_plugin (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_mpeg_block = {
+static const demuxer_info_t demux_info_mpeg_block = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_DEMUX, 26, "mpeg_block", XINE_VERSION_CODE, &demux_info_mpeg_block, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
