@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: demux_ogg.c,v 1.160.2.1 2005/04/20 17:21:05 mroi Exp $
+ * $Id: demux_ogg.c,v 1.174 2007/01/23 23:20:23 hadess Exp $
  *
  * demultiplexer for ogg streams
  *
@@ -43,17 +43,10 @@
 #include <vorbis/codec.h>
 
 #ifdef HAVE_SPEEX
-#ifdef HAVE_SPEEX_SUBDIR
 #include <speex/speex.h>
 #include <speex/speex_header.h>
 #include <speex/speex_stereo.h>
 #include <speex/speex_callbacks.h>
-#else
-#include <speex.h>
-#include <speex_header.h>
-#include <speex_stereo.h>
-#include <speex_callbacks.h>
-#endif
 #endif
 
 #ifdef HAVE_THEORA
@@ -62,6 +55,7 @@
 
 #define LOG_MODULE "demux_ogg"
 #define LOG_VERBOSE
+
 /*
 #define LOG
 */
@@ -75,6 +69,7 @@
 #include "xineutils.h"
 #include "demux.h"
 #include "bswap.h"
+#include "flacutils.h"
 
 #define CHUNKSIZE                8500
 #define PACKET_TYPE_HEADER       0x01
@@ -148,6 +143,7 @@ typedef struct demux_ogg_s {
 
   int                   num_audio_streams;
   int                   num_video_streams;
+  int                   unhandled_video_streams;
   int                   num_spu_streams;
 
   off_t                 avg_bitrate;
@@ -226,7 +222,10 @@ static int64_t get_pts (demux_ogg_t *this, int stream_num , int64_t granulepos )
     granuleshift = this->si[stream_num]->granuleshift;
     iframe = granulepos >> granuleshift;
     pframe = granulepos - (iframe << granuleshift);
-    return 1+((iframe+pframe) * this->si[stream_num]->factor / this->si[stream_num]->quotient);
+    if (this->si[stream_num]->quotient)
+      return 1+((iframe+pframe) * this->si[stream_num]->factor / this->si[stream_num]->quotient);
+    else
+      return 0;
   } else if (this->si[stream_num]->quotient)
     return 1+(granulepos * this->si[stream_num]->factor / this->si[stream_num]->quotient);
   else
@@ -537,7 +536,11 @@ static void update_chapter_display (demux_ogg_t *this, int stream_num, ogg_packe
     if (chapter >= 0) {
       char t_title[256];
 
-      snprintf(t_title, sizeof (t_title), "%s / %s", this->title, this->chapter_info->entries[chapter].name);
+      if (this->title) {
+        snprintf(t_title, sizeof (t_title), "%s / %s", this->title, this->chapter_info->entries[chapter].name);
+      } else {
+        snprintf(t_title, sizeof (t_title), "%s", this->chapter_info->entries[chapter].name);
+      }
       title = t_title;
     } else {
       title = this->title;
@@ -596,6 +599,7 @@ static void send_ogg_buf (demux_ogg_t *this,
     }
 
     if ((this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_AUDIO_SPEEX ||
+        (this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_AUDIO_FLAC ||
         (this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_AUDIO_VORBIS) {
       data = op->packet;
       size = op->bytes;
@@ -619,10 +623,9 @@ static void send_ogg_buf (demux_ogg_t *this,
              pts);
 
     _x_demux_send_data(this->audio_fifo, data, size,
-                       pts, this->si[stream_num]->buf_types, decoder_flags,
-                       normpos,
-                       pts / 90, this->time_length, 0);
-
+		       pts, this->si[stream_num]->buf_types, decoder_flags,
+		       normpos,
+		       pts / 90, this->time_length, 0);
 
 #ifdef HAVE_THEORA
   } else if ((this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_VIDEO_THEORA) {
@@ -1139,9 +1142,14 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
     this->num_video_streams++;
 
     this->si[stream_num]->factor = (int64_t) 90000 * (int64_t) this->t_info.fps_denominator;
+
+    if (!this->t_info.fps_numerator) {
+      this->t_info.fps_numerator = 1;   /* FIXME: default value ? */
+    }
     this->si[stream_num]->quotient = this->t_info.fps_numerator;
 
-    this->frame_duration = ((int64_t) 90000*this->t_info.fps_denominator)/this->t_info.fps_numerator;
+    this->frame_duration = ((int64_t) 90000*this->t_info.fps_denominator);
+    this->frame_duration /= this->t_info.fps_numerator;
 
     this->si[stream_num]->granuleshift = intlog(this->t_info.keyframe_frequency_force-1);
 
@@ -1151,13 +1159,15 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
     _x_meta_info_set(this->stream, XINE_META_INFO_VIDEOCODEC, "theora");
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_WIDTH, this->t_info.frame_width);
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT, this->t_info.frame_height);
-    _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION,
-                       ((int64_t) 90000 * this->t_info.fps_denominator) / this->t_info.fps_numerator);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION, this->frame_duration);
 
     /*currently aspect_nominator and -denumerator are 0?*/
-    if (this->t_info.aspect_denominator)
-      _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_RATIO,
-                         ((int64_t) this->t_info.aspect_numerator * 10000) / this->t_info.aspect_denominator);
+    if (this->t_info.aspect_denominator) {
+      int64_t ratio = ((int64_t) this->t_info.aspect_numerator * 10000);
+
+      ratio /= this->t_info.aspect_denominator;
+      _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_RATIO, ratio);
+    }
 
     lprintf ("decoded theora header \n");
     lprintf ("frameduration %d\n",this->frame_duration);
@@ -1172,10 +1182,75 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
   }
 #else
   this->si[stream_num]->buf_types = BUF_VIDEO_THEORA;
+  this->num_video_streams++;
+  this->unhandled_video_streams++;
   _x_meta_info_set(this->stream, XINE_META_INFO_VIDEOCODEC, "theora");
 #endif
 }
 
+static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
+  xine_flac_metadata_header header;
+  xine_flac_streaminfo_block streaminfo;
+  buf_element_t *buf;
+  xine_waveformatex wave;
+
+  /* Packet type */
+  _x_assert(op->packet[0] == 0x7F);
+
+  /* OggFLAC signature */
+  _x_assert(op->packet[1] == 'F'); _x_assert(op->packet[2] == 'L');
+  _x_assert(op->packet[3] == 'A'); _x_assert(op->packet[4] == 'C');
+
+  /* Version: supported only 1.0 */
+  _x_assert(op->packet[5] == 1); _x_assert(op->packet[6] == 0);
+
+  /* Header count */
+  this->si[stream_num]->headers = 0/*BE_16(&op->packet[7]) +1*/;
+
+  /* fLaC signature */
+  _x_assert(op->packet[9] == 'f'); _x_assert(op->packet[10] == 'L');
+  _x_assert(op->packet[11] == 'a'); _x_assert(op->packet[12] == 'C');
+
+  _x_parse_flac_metadata_header(&op->packet[13], &header);
+
+  switch ( header.blocktype ) {
+  case FLAC_BLOCKTYPE_STREAMINFO:
+    _x_assert(header.length == FLAC_STREAMINFO_SIZE);
+    _x_parse_flac_streaminfo_block(&op->packet[17], &streaminfo);
+    break;
+  }
+
+  this->si[stream_num]->buf_types = BUF_AUDIO_FLAC
+    +this->num_audio_streams++;
+
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, streaminfo.samplerate);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, streaminfo.channels);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS, streaminfo.bits_per_sample);
+
+  this->si[stream_num]->factor = 90000;
+
+  buf = this->audio_fifo->buffer_pool_alloc(this->audio_fifo);
+
+  buf->type = BUF_AUDIO_FLAC;
+  buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAME_END;
+
+  buf->decoder_info[0] = 0;
+  buf->decoder_info[1] = streaminfo.samplerate;
+  buf->decoder_info[2] = streaminfo.bits_per_sample;
+  buf->decoder_info[3] = streaminfo.channels;
+  buf->size = sizeof(xine_waveformatex) + FLAC_STREAMINFO_SIZE;
+  memcpy(buf->content+sizeof(xine_waveformatex), &op->packet[17], FLAC_STREAMINFO_SIZE);
+  xine_hexdump(&op->packet[17], FLAC_STREAMINFO_SIZE);
+  wave.cbSize = FLAC_STREAMINFO_SIZE;
+  memcpy(buf->content, &wave, sizeof(xine_waveformatex));
+
+  this->audio_fifo->put(this->audio_fifo, buf);
+
+  /* Skip the Ogg framing info */
+  op->bytes -= 9;
+  op->packet += 9;
+}
+
 static void decode_annodex_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
   lprintf ("Annodex stream detected\n");
   this->si[stream_num]->buf_types = BUF_CONTROL_NOP;
@@ -1222,16 +1297,16 @@ static void decode_anxdata_header (demux_ogg_t *this, const int stream_num, ogg_
     this->si[stream_num]->buf_types = BUF_AUDIO_VORBIS;
     this->num_audio_streams++;
   } else if (!strncmp(content_type, "audio/x-speex", content_type_length)) {
+    this->num_audio_streams++;
 #ifdef HAVE_SPEEX
     this->si[stream_num]->buf_types = BUF_AUDIO_SPEEX;
-    this->num_audio_streams++;
 #else
     this->si[stream_num]->buf_types = BUF_CONTROL_NOP;
 #endif
   } else if (!strncmp(content_type, "video/x-theora", content_type_length)) {
+    this->num_video_streams++;
 #ifdef HAVE_THEORA
     this->si[stream_num]->buf_types = BUF_VIDEO_THEORA;
-    this->num_video_streams++;
 #else
     this->si[stream_num]->buf_types = BUF_CONTROL_NOP;
 #endif
@@ -1322,6 +1397,8 @@ static void send_header (demux_ogg_t *this) {
           decode_text_header(this, stream_num, &op);
         } else if (!strncmp (&op.packet[1], "theora", 6)) {
           decode_theora_header(this, stream_num, &op);
+	} else if (!strncmp (&op.packet[1], "FLAC", 4)) {
+	  decode_flac_header(this, stream_num, &op);
         } else if (!strncmp (&op.packet[0], "Annodex", 7)) {
           decode_annodex_header(this, stream_num, &op);
         } else if (!strncmp (&op.packet[0], "AnxData", 7)) {
@@ -1436,7 +1513,7 @@ static int demux_ogg_send_chunk (demux_plugin_t *this_gen) {
     /* printf("demux_ogg:   got a packet\n"); */
 
     if ((*op.packet & PACKET_TYPE_HEADER) &&
-        (this->si[stream_num]->buf_types!=BUF_VIDEO_THEORA) && (this->si[stream_num]->buf_types!=BUF_AUDIO_SPEEX)) {
+        (this->si[stream_num]->buf_types!=BUF_VIDEO_THEORA) && (this->si[stream_num]->buf_types!=BUF_AUDIO_SPEEX) && (this->si[stream_num]->buf_types!=BUF_AUDIO_FLAC)) {
       if (op.granulepos != -1) {
         this->si[stream_num]->header_granulepos = op.granulepos;
         lprintf ("header with granulepos, remembering granulepos\n");
@@ -1533,6 +1610,7 @@ static int demux_ogg_send_chunk (demux_plugin_t *this_gen) {
       this->num_streams       = 0;
       this->num_audio_streams = 0;
       this->num_video_streams = 0;
+      this->unhandled_video_streams = 0;
       this->num_spu_streams   = 0;
       this->avg_bitrate       = 1;
       
@@ -1626,6 +1704,8 @@ static void demux_ogg_send_headers (demux_plugin_t *this_gen) {
 
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO,
                        this->num_video_streams > 0);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HANDLED,
+		     this->num_video_streams > this->unhandled_video_streams);
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO,
                        this->num_audio_streams > 0);
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_MAX_SPU_CHANNEL,
@@ -1850,7 +1930,7 @@ static int detect_ogg_content (int detection_method, demux_class_t *class_gen,
     }
 
     case METHOD_BY_EXTENSION: {
-      char *extensions, *mrl;
+      const char *extensions, *mrl;
 
       mrl = input->get_mrl (input);
       extensions = class_gen->get_extensions (class_gen);
@@ -1912,7 +1992,7 @@ static int detect_anx_content (int detection_method, demux_class_t *class_gen,
 #undef ANNODEX_SIGNATURE_SEARCH
 
     case METHOD_BY_EXTENSION: {
-      char *extensions, *mrl;
+      const char *extensions, *mrl;
 
       mrl = input->get_mrl (input);
       extensions = class_gen->get_extensions (class_gen);
@@ -2023,19 +2103,19 @@ static demux_plugin_t *ogg_open_plugin (demux_class_t *class_gen,
  * Annodex demuxer class
  */
 
-static char *anx_get_description (demux_class_t *this_gen) {
+static const char *anx_get_description (demux_class_t *this_gen) {
   return "Annodex demux plugin";
 }
  
-static char *anx_get_identifier (demux_class_t *this_gen) {
+static const char *anx_get_identifier (demux_class_t *this_gen) {
   return "Annodex";
 }
 
-static char *anx_get_extensions (demux_class_t *this_gen) {
+static const char *anx_get_extensions (demux_class_t *this_gen) {
   return "anx axa axv";
 }
 
-static char *anx_get_mimetypes (demux_class_t *this_gen) {
+static const char *anx_get_mimetypes (demux_class_t *this_gen) {
   return "application/x-annodex: ogg: Annodex media;";
 }
 
@@ -2064,22 +2144,23 @@ static void *anx_init_class (xine_t *xine, void *data) {
  * ogg demuxer class
  */
 
-static char *ogg_get_description (demux_class_t *this_gen) {
+static const char *ogg_get_description (demux_class_t *this_gen) {
   return "OGG demux plugin";
 }
  
-static char *ogg_get_identifier (demux_class_t *this_gen) {
+static const char *ogg_get_identifier (demux_class_t *this_gen) {
   return "OGG";
 }
 
-static char *ogg_get_extensions (demux_class_t *this_gen) {
+static const char *ogg_get_extensions (demux_class_t *this_gen) {
   return "ogg ogm spx";
 }
 
-static char *ogg_get_mimetypes (demux_class_t *this_gen) {
+static const char *ogg_get_mimetypes (demux_class_t *this_gen) {
   return "audio/x-ogg: ogg: OggVorbis Audio;"
          "audio/x-speex: ogg: Speex Audio;"
-         "application/x-ogg: ogg: OggVorbis Audio;";
+         "application/x-ogg: ogg: Ogg Stream;"
+         "application/ogg: ogg: Ogg Stream;";
 }
 
 static void ogg_class_dispose (demux_class_t *this_gen) {
@@ -2106,15 +2187,15 @@ static void *ogg_init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_anx = {
+static const demuxer_info_t demux_info_anx = {
   20                       /* priority */
 };
 
-demuxer_info_t demux_info_ogg = {
+static const demuxer_info_t demux_info_ogg = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_DEMUX, 26, "ogg", XINE_VERSION_CODE, &demux_info_ogg, ogg_init_class },
   { PLUGIN_DEMUX, 26, "anx", XINE_VERSION_CODE, &demux_info_anx, anx_init_class },
