@@ -20,7 +20,7 @@
  * Compact Disc Digital Audio (CDDA) Input Plugin 
  *   by Mike Melanson (melanson@pcisys.net)
  *
- * $Id: input_cdda.c,v 1.80 2005/11/15 13:29:10 miguelfreitas Exp $
+ * $Id: input_cdda.c,v 1.93 2007/01/19 01:05:24 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,7 @@
 #include <unistd.h>
 
 #include <sys/types.h>
-#ifdef SYS_PARAM_H
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 #ifdef HAVE_DIRENT_H
@@ -111,7 +111,7 @@ typedef struct _cdrom_toc {
  *************************************************************************/
 
 #define MAX_TRACKS     99
-#define CACHED_FRAMES  100
+#define CACHED_FRAMES  500
 
 typedef struct {
   int                  start;
@@ -908,7 +908,11 @@ static int parse_url (char *urlbuf, char** host, int *port) {
 }
 #endif
 
-static int network_command( xine_stream_t *stream, int socket, char *data_buf, char *msg, ...)
+static int
+#ifdef __GNUC__
+__attribute__((format (printf, 4, 5)))
+#endif
+network_command( xine_stream_t *stream, int socket, char *data_buf, char *msg, ...)
 {
   char     buf[_BUFSIZ];
   va_list  args;
@@ -1393,73 +1397,78 @@ static int _cdda_load_cached_cddb_infos(cdda_input_plugin_t *this) {
 	  return 0;
 	}
 	else {
-	  char buffer[256], *ln, *pt;
+	  char buffer[256], *ln;
 	  char buf[256];
 	  int  tnum;
 	  char *dtitle = NULL;
 	  
-	  while((ln = fgets(buffer, 255, fd)) != NULL) {
+	  while ((ln = fgets(buffer, 255, fd)) != NULL) {
 
 	    buffer[strlen(buffer) - 1] = '\0';
 	    
-	    if(sscanf(buffer, "DTITLE=%s", &buf[0]) == 1) {
-	      char *artist, *title;
+	    if (sscanf(buffer, "DTITLE=%s", &buf[0]) == 1) {
+	      char *pt, *artist, *title;
 
 	      pt = strrchr(buffer, '=');
-	      if(pt)
+	      if (pt) {
 		pt++;
 
-	      if (dtitle != NULL)
-	      {
-		dtitle = (char *) realloc(dtitle, strlen(dtitle) + strlen(pt) + 1);
-		strcat(dtitle,pt);
-		pt = dtitle;
-	      }
-	      dtitle = strdup(pt);
+	        if (dtitle != NULL)
+	        {
+		  dtitle = (char *) realloc(dtitle, strlen(dtitle) + strlen(pt) + 1);
+		  strcat(dtitle, pt);
+		  pt = dtitle;
+	        }
+	        dtitle = strdup(pt);
  
-	      artist = pt;
-	      title = strstr(pt, " / ");
-	      if(title) {
-		*title++ = '\0';
-		title += 2;
-	      }
-	      else {
-		title = artist;
-		artist = NULL;
-	      }
+	        artist = pt;
+	        title = strstr(pt, " / ");
+	        if (title) {
+		  *title++ = '\0';
+		  title += 2;
+	        }
+	        else {
+		  title = artist;
+		  artist = NULL;
+	        }
 
-	      if(artist)
-		this->cddb.disc_artist = strdup(artist);
+	        if (artist)
+		  this->cddb.disc_artist = strdup(artist);
 
-	      this->cddb.disc_title = strdup(title);
+	        this->cddb.disc_title = strdup(title);
+	      }
 	    }
-	    else if(sscanf(buffer, "DYEAR=%s", &buf[0]) == 1) {
+	    else if (sscanf(buffer, "DYEAR=%s", &buf[0]) == 1) {
+	      char *pt;
+
 	      pt = strrchr(buffer, '=');
 	      pt++;
 	      if (pt != NULL && strlen(pt) == 4)
 		this->cddb.disc_year = strdup(pt);
 	    }
-	    else if(sscanf(buffer, "TTITLE%d=%s", &tnum, &buf[0]) == 2) {
+	    else if (sscanf(buffer, "TTITLE%d=%s", &tnum, &buf[0]) == 2) {
+	      char *pt;
+
 	      pt = strrchr(buffer, '=');
-	      if(pt)
+	      if (pt)
 		pt++;
 	      if (this->cddb.track[tnum].title == NULL)
 		this->cddb.track[tnum].title = strdup(pt); 
 	      else
 	      {
 		this->cddb.track[tnum].title
-		  = (char *) realloc(this->cddb.track[tnum].title,strlen(this->cddb.track[tnum].title) + strlen(pt) + 1);
-		strcat(this->cddb.track[tnum].title,pt);
+		  = (char *) realloc(this->cddb.track[tnum].title, strlen(this->cddb.track[tnum].title) + strlen(pt) + 1);
+		strcat(this->cddb.track[tnum].title, pt);
 	      }
 	    }
 	    else {
-	      if(!strncmp(buffer, "EXTD=", 5)) {
+	      if (!strncmp(buffer, "EXTD=", 5)) {
 		char *y;
 		int   nyear;
 		
 		y = strstr(buffer, "YEAR:");
 		if(y) {
-		  if(sscanf(y+5, "%4d", &nyear) == 1) {
+		  if (sscanf(y+5, "%4d", &nyear) == 1) {
 		    char year[5];
 
 		    snprintf(year, 5, "%d", nyear);
@@ -1556,6 +1565,7 @@ static void _cdda_cddb_socket_close(cdda_input_plugin_t *this) {
 static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
   cdda_input_class_t *this_class = (cdda_input_class_t *)this->class;
   char buffer[2048], buffercache[32768], *m, *p;
+  char *dtitle = NULL;
   int err, i;
 
   if(this == NULL) {
@@ -1729,12 +1739,11 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
 			
     this->cddb.have_cddb_info = 1;
     memset(&buffercache, 0, sizeof(buffercache));
-						
+
     while (strcmp(buffer, ".")) {
       char buf[2048];
       int tnum;
       int bufsize = strlen(buffercache);
-      char *dtitle = NULL;
 
       memset(&buffer, 0, sizeof(buffer));
       _cdda_cddb_socket_read(this, buffer, sizeof(buffer) - 1);
@@ -1750,7 +1759,7 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
           if (dtitle != NULL)
           {
             dtitle = (char *) realloc(dtitle, strlen(dtitle) + strlen(pt) + 1);
-            strcat(dtitle,pt);
+            strcat(dtitle, pt);
             pt = dtitle;
           }
           dtitle = strdup(pt);
@@ -1791,8 +1800,8 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
           else
           {
             this->cddb.track[tnum].title
-              = (char *) realloc(this->cddb.track[tnum].title,strlen(this->cddb.track[tnum].title) + strlen(pt) + 1);
-            strcat(this->cddb.track[tnum].title,pt);
+              = (char *) realloc(this->cddb.track[tnum].title, strlen(this->cddb.track[tnum].title) + strlen(pt) + 1);
+            strcat(this->cddb.track[tnum].title, pt);
           }
         }
       }
@@ -1813,9 +1822,9 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
           }
         }
       }
-      free(dtitle);
     }
-
+    free(dtitle);
+    
     /* Save cddb info and close socket */
     _cdda_save_cached_cddb_infos(this, buffercache);
     _cdda_cddb_socket_close(this);
@@ -2166,11 +2175,20 @@ static uint32_t cdda_plugin_get_capabilities (input_plugin_t *this_gen) {
 
 static off_t cdda_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
 
+  /* only allow reading in block-sized chunks */
+
+  return 0;
+}
+
+static buf_element_t *cdda_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, 
+  off_t nlen) {
+
   cdda_input_plugin_t *this = (cdda_input_plugin_t *) this_gen;
+  buf_element_t *buf;
+  unsigned char frame_data[CD_RAW_FRAME_SIZE];
   int err = 0;
 
-  /* only allow reading in block-sized chunks */
-  if (len != CD_RAW_FRAME_SIZE)
+  if (nlen != CD_RAW_FRAME_SIZE)
     return 0;
 
   if (this->current_frame > this->last_frame)
@@ -2204,26 +2222,14 @@ static off_t cdda_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
   if( err < 0 )
     return 0;
     
-  memcpy(buf, this->cache[this->current_frame-this->cache_first], CD_RAW_FRAME_SIZE);
+  memcpy(frame_data, this->cache[this->current_frame-this->cache_first], CD_RAW_FRAME_SIZE);
   this->current_frame++;
 
-  return CD_RAW_FRAME_SIZE;
-}
-
-static buf_element_t *cdda_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, 
-  off_t nlen) {
-
-  buf_element_t *buf;
-
   buf = fifo->buffer_pool_alloc(fifo);
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
-
-  buf->size = cdda_plugin_read(this_gen, buf->content, nlen);
-  if (buf->size == 0) {
-    buf->free_buffer(buf);
-    buf = NULL;
-  }
+  buf->size = CD_RAW_FRAME_SIZE;
+  memcpy(buf->mem, frame_data, CD_RAW_FRAME_SIZE);
 
   return buf;
 }
@@ -2264,7 +2270,7 @@ static uint32_t cdda_plugin_get_blocksize (input_plugin_t *this_gen) {
   return CD_RAW_FRAME_SIZE;
 }
 
-static char* cdda_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* cdda_plugin_get_mrl (input_plugin_t *this_gen) {
   cdda_input_plugin_t *this = (cdda_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -2462,6 +2468,8 @@ static char ** cdda_class_get_autoplay_list (input_class_t *this_gen,
      * let's create a dummy instance in such a condition */
     ip = (cdda_input_plugin_t *)xine_xmalloc(sizeof(cdda_input_plugin_t));
     ip->stream = NULL;
+    ip->fd = -1;
+    ip->net_fd = -1;
   }
 
 #ifndef WIN32
@@ -2529,28 +2537,33 @@ static input_plugin_t *cdda_class_get_instance (input_class_t *cls_gen, xine_str
   /* fetch the CD track to play */
   if (!strncasecmp (mrl, "cdda:/", 6)) {
 
-    if ( strlen(mrl) > 8 && strchr(&mrl[8],'/') ) {
-      int i;
-
-      cdda_device = strdup(&mrl[6]);
-
-      i = strlen(cdda_device)-1;
-      while( i && cdda_device[i] != '/' )
-        i--;
-
-      if( i ) {
-        cdda_device[i] = '\0';
-        track = atoi(&cdda_device[i+1]);
-      } else
-        track = -1;        
-
+    const char *p, *slash = mrl + 6;
+    while (*slash == '/')
+      ++slash;
+    p = --slash; /* point at a slash */
+    while (*p >= '0' && *p <= '9')
+      ++p;
+    if (*p) {
+      char *lastslash;
+      cdda_device = strdup (slash);
+      p = lastslash = strrchr (cdda_device, '/'); /* guaranteed to return non-NULL */
+      while (*++p >= '0' && *p <= '9')
+        /**/;
+      if (!*p) {
+        track = atoi (lastslash + 1);
+        *lastslash = 0;
+        if (lastslash == cdda_device) {
+          free (cdda_device);
+          cdda_device = NULL;
+        }
+      } else {
+        track = -1;
+      }
     } else {
-      track = atoi(&mrl[6]);
+      track = atoi (slash + 1);
     }
-    
-    /* CD tracks start at 1, reject illegal tracks */
-    if (track <= 0)
-      return NULL;
+    if (track < 1)
+      track = 1;
   } else
     return NULL;
 
@@ -2610,7 +2623,7 @@ static const char *cdda_class_get_identifier (input_class_t *this_gen) {
   return "cdda";
 }
 
-static char *cdda_class_get_description (input_class_t *this_gen) {
+static const char *cdda_class_get_description (input_class_t *this_gen) {
   return _("CD Digital Audio (aka. CDDA)");
 }
 
@@ -2662,8 +2675,8 @@ static void *init_plugin (xine_t *xine, void *data) {
   this->mrls_allocated_entries = 0;
   this->ip = NULL;
   
-  this->cdda_device = config->register_string(config, "media.audio_cd.device", 
-					      DEFAULT_CDDA_DEVICE,
+  this->cdda_device = config->register_filename(config, "media.audio_cd.device", 
+					      DEFAULT_CDDA_DEVICE, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 					      _("device used for CD audio"),
 					      _("The path to the device, usually a "
 						"CD or DVD drive, which you intend to use "
@@ -2691,8 +2704,8 @@ static void *init_plugin (xine_t *xine, void *data) {
 		       "title and track information from."), XINE_CONFIG_SECURITY,
 		       port_changed_cb, (void *) this);
   
-  config->register_string(config, "media.audio_cd.cddb_cachedir", 
-			  (_cdda_cddb_get_default_location()),
+  config->register_filename(config, "media.audio_cd.cddb_cachedir", 
+			  (_cdda_cddb_get_default_location()), XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
 			  _("CDDB cache directory"), _("The replies from the CDDB server will be "
 			  "cached in this directory.\nThis setting is security critical, because files "
 			  "with uncontrollable names will be created in this directory. Be sure to use "
@@ -2717,7 +2730,7 @@ static void *init_plugin (xine_t *xine, void *data) {
   return this;
 }
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "CD", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
