@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project, 
+ * Copyright (C) 2000-2005 the xine project, 
  *                         Rich Wareham <richwareham@users.sourceforge.net>
  * 
  * This file is part of xine, a free video player.
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_dvd.c,v 1.201 2005/07/11 11:35:41 hadess Exp $
+ * $Id: input_dvd.c,v 1.214 2007/01/19 01:05:25 dgp85 Exp $
  *
  */
 
@@ -47,7 +47,6 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <dirent.h>
 
 #include <unistd.h>
 #include <fcntl.h>
@@ -896,6 +895,20 @@ static off_t dvd_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
   return dvd_plugin_get_current_pos(this_gen);
 }
 
+static off_t dvd_plugin_seek_time (input_plugin_t *this_gen, int time_offset, int origin) {
+  dvd_input_plugin_t *this = (dvd_input_plugin_t*)this_gen;
+ 
+  trace_print("Called\n");
+  
+  if(!this || !this->dvdnav || origin != SEEK_SET) {
+    return -1;
+  }
+ 
+  dvdnav_time_search(this->dvdnav, time_offset * 90);
+  
+  return dvd_plugin_get_current_pos(this_gen);
+}
+
 static off_t dvd_plugin_get_length (input_plugin_t *this_gen) {
   dvd_input_plugin_t *this = (dvd_input_plugin_t*)this_gen;
   uint32_t pos=0;
@@ -917,7 +930,7 @@ static uint32_t dvd_plugin_get_blocksize (input_plugin_t *this_gen) {
   return DVD_BLOCK_SIZE;
 }
 
-static char* dvd_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* dvd_plugin_get_mrl (input_plugin_t *this_gen) {
   dvd_input_plugin_t *this = (dvd_input_plugin_t*)this_gen;
   
   trace_print("Called\n");
@@ -986,7 +999,8 @@ static void dvd_handle_events(dvd_input_plugin_t *this) {
       break;
     case XINE_EVENT_INPUT_MENU3:
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_dvd: MENU3 key hit.\n");
-      dvdnav_menu_call(this->dvdnav, DVD_MENU_Root);
+      if (dvdnav_menu_call(this->dvdnav, DVD_MENU_Root) == DVDNAV_STATUS_ERR)
+        dvdnav_menu_call(this->dvdnav, DVD_MENU_Title);
       break;
     case XINE_EVENT_INPUT_MENU4:
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_dvd: MENU4 key hit.\n");
@@ -1534,7 +1548,7 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
 
     tt = strtol(title_part, NULL, 10);
     dvdnav_get_number_of_titles(this->dvdnav, &titles);
-    if((tt <= 0) || (tt > titles)) {
+    if((tt < 0) || (tt > titles)) {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
 	      "input_dvd: Title %i is out of range (1 to %i).\n", tt, titles);
       dvdnav_close(this->dvdnav);
@@ -1547,7 +1561,7 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
     if(delimiter) {
       pr = strtol(delimiter+1, NULL, 10);
       dvdnav_get_number_of_parts(this->dvdnav, tt, &parts);
-      if ((pr <= 0) || (pr > parts)) {
+      if ((pr < 0) || (pr > parts)) {
 	xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
 		"input_dvd: Part %i is out of range (1 to %i).\n", pr, parts);
 	dvdnav_close(this->dvdnav);
@@ -1559,11 +1573,13 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
 #ifdef INPUT_DEBUG
     printf("input_dvd: Jumping to TT >%i<, PTT >%i<\n", tt, pr);
 #endif
-    if(pr != -1) {
-      dvdnav_part_play(this->dvdnav, tt, pr);
-    } else {
-      dvdnav_title_play(this->dvdnav, tt);
-    }
+    if (tt > 0) {
+      if (pr > 0)
+        dvdnav_part_play(this->dvdnav, tt, pr);
+      else
+        dvdnav_title_play(this->dvdnav, tt);
+    } else
+      this->mode = MODE_NAVIGATE;
   }
 #ifdef INPUT_DEBUG
   printf("input_dvd: DVD device successfully opened.\n");
@@ -1628,6 +1644,7 @@ static input_plugin_t *dvd_class_get_instance (input_class_t *class_gen, xine_st
   this->input_plugin.read               = dvd_plugin_read;
   this->input_plugin.read_block         = dvd_plugin_read_block;
   this->input_plugin.seek               = dvd_plugin_seek;
+  this->input_plugin.seek_time          = dvd_plugin_seek_time;
   this->input_plugin.get_current_pos    = dvd_plugin_get_current_pos;
   this->input_plugin.get_length         = dvd_plugin_get_length;
   this->input_plugin.get_blocksize      = dvd_plugin_get_blocksize;
@@ -1663,7 +1680,7 @@ static input_plugin_t *dvd_class_get_instance (input_class_t *class_gen, xine_st
   return &this->input_plugin;
 }
 
-static char *dvd_class_get_description (input_class_t *this_gen) {
+static const char *dvd_class_get_description (input_class_t *this_gen) {
   trace_print("Called\n");
 
   return "DVD Navigator";
@@ -1709,7 +1726,15 @@ static char **dvd_class_get_autoplay_list (input_class_t *this_gen,
 
 static void dvd_class_dispose(input_class_t *this_gen) {
   dvd_input_class_t *this = (dvd_input_class_t*)this_gen;
-  
+  config_values_t *config = this->xine->config;
+
+  config->unregister_callback(config, "media.dvd.device");
+  config->unregister_callback(config, "media.dvd.region");
+  config->unregister_callback(config, "media.dvd.language");
+  config->unregister_callback(config, "media.dvd.readahead");
+  config->unregister_callback(config, "media.dvd.seek_behaviour");
+  config->unregister_callback(config, "media.dvd.play_single_chapter");
+
   free(this->eject_device);
   free(this);
 }
@@ -1750,26 +1775,33 @@ static void *init_class (xine_t *xine, void *data) {
   this->input_class.eject_media        = dvd_class_eject_media;
   
   this->config                         = config;
+  this->xine                           = xine;
 
   this->ip                             = NULL;
 
-  this->dvd_device = config->register_string(config,
+  this->dvd_device = config->register_filename(config,
 					     "media.dvd.device",
-					     DVD_PATH,
+					     DVD_PATH, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 					     _("device used for DVD playback"),
 					     _("The path to the device, usually a "
 					       "DVD drive, which you intend to use for playing DVDs."),
 					     10, device_change_cb, (void *)this);
-  
-  if ((dvdcss = dlopen("libdvdcss.so.2", RTLD_LAZY)) != NULL) {
+
+#ifdef HOST_OS_DARWIN
+  if ((dvdcss = dlopen("libdvdcss.2.dylib", RTLD_LAZY)) != NULL)
+#else
+  if ((dvdcss = dlopen("libdvdcss.so.2", RTLD_LAZY)) != NULL)
+#endif
+  {
     /* we have found libdvdcss, enable the specific config options */
     char *raw_device;
     static char *decrypt_modes[] = { "key", "disc", "title", NULL };
     char *css_cache_default, *css_cache;
     int mode;
     
-    raw_device = config->register_string(config, "media.dvd.raw_device",
-					 RDVD_PATH, _("raw device set up for DVD access"),
+    raw_device = config->register_filename(config, "media.dvd.raw_device",
+					 RDVD_PATH, XINE_CONFIG_STRING_IS_DEVICE_NAME,
+					 _("raw device set up for DVD access"),
 					 _("If this points to a raw device connected to your "
 					   "DVD device, xine will use the raw device for playback. "
 					   "This has the advantage of being slightly faster and "
@@ -1791,7 +1823,7 @@ static void *init_class (xine_t *xine, void *data) {
     
     css_cache_default = (char *)malloc(strlen(xine_get_homedir()) + 10);
     sprintf(css_cache_default, "%s/.dvdcss/", xine_get_homedir());
-    css_cache = config->register_string(config, "media.dvd.css_cache_path", css_cache_default,
+    css_cache = config->register_filename(config, "media.dvd.css_cache_path", css_cache_default, XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
 					_("path to the title key cache"),
 					_("Since cracking the copy protection of scrambled DVDs can "
 					  "be quite time consuming, libdvdcss will cache the cracked "
@@ -1883,8 +1915,8 @@ static void *init_class (xine_t *xine, void *data) {
 }
 
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "DVD", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "DVD", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
