@@ -19,7 +19,7 @@
  *
  * input plugin for http network streams
  *
- * $Id: input_http.c,v 1.124 2006/11/30 10:54:18 dgp85 Exp $
+ * $Id: input_http.c,v 1.127 2007/01/19 01:05:25 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -335,12 +335,9 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
           /* prepares the event */
           radio = _x_meta_info_get(this->stream, XINE_META_INFO_ALBUM);
           if (radio) {
-            int len = strlen(radio);
-            strncpy(data.str, radio, sizeof(data.str));
-            strncat(data.str, " - ", sizeof(data.str) - len);
-            strncat(data.str, songtitle, sizeof(data.str) - len - 3);
+	    snprintf (data.str, sizeof(data.str), "%s - %s", radio, songtitle);
           } else {
-            strncpy(data.str, songtitle, sizeof(data.str));
+            strncpy(data.str, songtitle, sizeof(data.str)-1);
           }
           data.str[sizeof(data.str) - 1] = '\0';
           data.str_len = strlen(data.str) + 1;
@@ -577,7 +574,7 @@ static off_t http_plugin_seek(input_plugin_t *this_gen, off_t offset, int origin
   return this->curpos;
 }
 
-static char* http_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* http_plugin_get_mrl (input_plugin_t *this_gen) {
   http_input_plugin_t *this = (http_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -766,13 +763,11 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   }
   
   snprintf(this->buf + buflen, BUFSIZE - buflen,
-           "User-Agent: xine/%s\015\012", VERSION);
-  buflen = strlen(this->buf);
-  strncat (this->buf, "Accept: */*\015\012", BUFSIZE - buflen);
-  buflen = strlen(this->buf);
-  strncat (this->buf, "Icy-MetaData: 1\015\012", BUFSIZE - buflen);
-  buflen = strlen(this->buf);
-  strncat (this->buf, "\015\012", BUFSIZE - buflen);
+           "User-Agent: xine/%s\015\012"
+           "Accept: */*\015\012"
+           "Icy-MetaData: 1\015\012"
+           "\015\012",
+           VERSION);
   buflen = strlen(this->buf);
   if (_x_io_tcp_write (this->stream, this->fh, this->buf, buflen) != buflen) {
     _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "couldn't send request", NULL);
@@ -995,7 +990,7 @@ static input_plugin_t *http_class_get_instance (input_class_t *cls_gen, xine_str
   return &this->input_plugin;
 }
 
-static char *http_class_get_description (input_class_t *this_gen) {
+static const char *http_class_get_description (input_class_t *this_gen) {
   return _("http input plugin");
 }
 
