@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_mms.c,v 1.56 2005/02/07 23:58:58 tmattern Exp $
+ * $Id: input_mms.c,v 1.68 2007/01/19 01:05:25 dgp85 Exp $
  *
  * mms input plugin based on work from major mms
  */
@@ -55,10 +55,6 @@
 #define PROTOCOL_MMST      1
 #define PROTOCOL_MMSH      2
 
-#if !defined(NDELAY) && defined(O_NDELAY)
-#define FNDELAY O_NDELAY
-#endif
-
 /* network bandwidth */
 const uint32_t mms_bandwidths[]={14400,19200,28800,33600,34430,57600,
                                   115200,262200,393216,524300,1544000,10485800};
@@ -131,7 +127,7 @@ static buf_element_t *mms_plugin_read_block (input_plugin_t *this_gen,
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
   
-  total_bytes = mms_plugin_read (this_gen, buf->content, todo);
+  total_bytes = mms_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -205,6 +201,28 @@ static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
   return curpos;
 }
 
+static off_t mms_plugin_seek_time (input_plugin_t *this_gen, int time_offset, int origin) {
+  mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
+  off_t               curpos = 0;
+
+  lprintf ("seek_time %d msec, origin %d\n", time_offset, origin);
+  
+  switch (this->protocol) {
+    case PROTOCOL_MMST:
+      if (origin == SEEK_SET)
+        mms_set_start_time (this->mms, time_offset);
+      curpos = mms_get_current_pos (this->mms);
+      break;
+    case PROTOCOL_MMSH:
+      if (origin == SEEK_SET)
+        mmsh_set_start_time (this->mmsh, time_offset);
+      curpos = mmsh_get_current_pos (this->mmsh);
+      break;
+  }
+    
+  return curpos;
+}
+
 static off_t mms_plugin_get_length (input_plugin_t *this_gen) {
   mms_input_plugin_t   *this = (mms_input_plugin_t *) this_gen; 
   off_t                 length = 0;
@@ -274,7 +292,7 @@ static void mms_plugin_dispose (input_plugin_t *this_gen) {
   free (this);
 }
 
-static char* mms_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* mms_plugin_get_mrl (input_plugin_t *this_gen) {
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -404,6 +422,7 @@ static input_plugin_t *mms_class_get_instance (input_class_t *cls_gen, xine_stre
   this->input_plugin.read              = mms_plugin_read;
   this->input_plugin.read_block        = mms_plugin_read_block;
   this->input_plugin.seek              = mms_plugin_seek;
+  this->input_plugin.seek_time         = mms_plugin_seek_time;
   this->input_plugin.get_current_pos   = mms_plugin_get_current_pos;
   this->input_plugin.get_length        = mms_plugin_get_length;
   this->input_plugin.get_blocksize     = mms_plugin_get_blocksize;
@@ -420,7 +439,7 @@ static input_plugin_t *mms_class_get_instance (input_class_t *cls_gen, xine_stre
  * mms input plugin class stuff
  */
 
-static char *mms_class_get_description (input_class_t *this_gen) {
+static const char *mms_class_get_description (input_class_t *this_gen) {
   return _("mms streaming input plugin");
 }
 
@@ -431,6 +450,10 @@ static const char *mms_class_get_identifier (input_class_t *this_gen) {
 static void mms_class_dispose (input_class_t *this_gen) {
   mms_input_class_t  *this = (mms_input_class_t *) this_gen;
 
+  this->xine->config->unregister_callback(this->xine->config,
+		  			  "media.network.bandwidth");
+  this->xine->config->unregister_callback(this->xine->config,
+					  "media.network.mms_protocol");
   free (this);
 }
 
@@ -475,8 +498,8 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "mms", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "mms", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
