@@ -803,7 +803,8 @@ static int open_video_capture_device(v4l_input_plugin_t *this)
 {
   int          found       = 0;
   int          tuner_found = 0;
-  int          i, j, ret;
+  int          i, ret;
+  unsigned int j;
   cfg_entry_t *entry;
   
   lprintf("open_video_capture_device\n");
@@ -1627,7 +1628,7 @@ static void v4l_plugin_dispose (input_plugin_t *this_gen) {
  *
  * Get the current MRL used by the plugin.
  */
-static char* v4l_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* v4l_plugin_get_mrl (input_plugin_t *this_gen) {
   v4l_input_plugin_t *this = (v4l_input_plugin_t *) this_gen;
   
   return this->mrl;
@@ -1906,8 +1907,8 @@ static void *init_video_class (xine_t *xine, void *data)
   this->input_class.dispose            = v4l_class_dispose;
   this->input_class.eject_media        = NULL;
   
-  config->register_string (config, "media.video4linux.video_device",
-			   VIDEO_DEV,
+  config->register_filename (config, "media.video4linux.video_device",
+			   VIDEO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 			   _("v4l video device"),
 			   _("The path to your Video4Linux video device."),
 			   10, NULL, NULL);
@@ -1932,8 +1933,8 @@ static void *init_radio_class (xine_t *xine, void *data)
   this->input_class.dispose            = v4l_class_dispose;
   this->input_class.eject_media        = NULL;
   
-  config->register_string (config, "media.video4linux.radio_device",
-			   RADIO_DEV,
+  config->register_filename (config, "media.video4linux.radio_device",
+			   RADIO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 			   _("v4l radio device"),
 			   _("The path to your Video4Linux radio device."),
 			   10, NULL, NULL);
@@ -1945,7 +1946,7 @@ static void *init_radio_class (xine_t *xine, void *data)
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "v4l_radio", XINE_VERSION_CODE, NULL, init_radio_class },
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "v4l_tv", XINE_VERSION_CODE, NULL, init_video_class },
