@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_vcd.c,v 1.79 2005/02/07 23:58:58 tmattern Exp $
+ * $Id: input_vcd.c,v 1.88 2007/01/19 01:05:25 dgp85 Exp $
  *
  */
 
@@ -25,6 +25,7 @@
 #include "config.h"
 #endif
 
+#include <netinet/in.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -622,6 +623,7 @@ static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
 }
 #endif
 
+#if defined (__linux__) || defined(__sun) || defined(HOST_OS_DARWIN)
 static off_t vcd_time_to_offset (int min, int sec, int frame) {
   return min * 60 * 75 + sec * 75 + frame;
 }
@@ -636,7 +638,6 @@ static void vcd_offset_to_time (off_t offset, uint8_t *min, uint8_t *sec,
 
 }
 
-#if defined (__linux__) || defined(__sun) || defined(HOST_OS_DARWIN)
 static off_t vcd_plugin_seek (input_plugin_t *this_gen, 
 			      off_t offset, int origin) {
 
@@ -812,7 +813,7 @@ static void vcd_plugin_dispose (input_plugin_t *this_gen ) {
   free (this);
 }
 
-static char* vcd_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* vcd_plugin_get_mrl (input_plugin_t *this_gen) {
   vcd_input_plugin_t *this = (vcd_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -918,7 +919,7 @@ static input_plugin_t *vcd_class_get_instance (input_class_t *cls_gen, xine_stre
  * vcd input plugin class stuff
  */
 
-static char *vcd_class_get_description (input_class_t *this_gen) {
+static const char *vcd_class_get_description (input_class_t *this_gen) {
   return _("Video CD input plugin");
 }
 
@@ -930,6 +931,9 @@ static void vcd_class_dispose (input_class_t *this_gen) {
 
   vcd_input_class_t  *this = (vcd_input_class_t *) this_gen;
   int                 i;
+  config_values_t *config = this->xine->config;
+
+  config->unregister_callback(config, "media.vcd.device");
 
   for (i = 0; i < 100; i++)
     free (this->filelist[i]);
@@ -1090,7 +1094,7 @@ static void *init_class (xine_t *xine, void *data) {
   this->input_class.dispose            = vcd_class_dispose;
   this->input_class.eject_media        = vcd_class_eject_media;
 
-  this->device = config->register_string (config, "media.vcd.device", CDROM,
+  this->device = config->register_filename (config, "media.vcd.device", CDROM, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 					  _("device used for VCD playback"),
 					  _("The path to the device, usually a CD or DVD drive, "
 					    "you intend to play your VideoCDs with."),
@@ -1110,8 +1114,8 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "VCDO", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "VCDO", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
