@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_decoder.c,v 1.60 2006/06/18 20:29:04 dgp85 Exp $
+ * $Id: video_decoder.c,v 1.69 2007/01/21 15:12:21 klan Exp $
  *
  * xine video decoder plugin using ffmpeg
  *
@@ -25,6 +25,7 @@
  
 #ifdef HAVE_CONFIG_H
 #include "config.h"
+#include "ffmpeg_config.h"
 #endif
 
 #include <stdlib.h>
@@ -113,7 +114,11 @@ struct ff_video_decoder_s {
   int               yuv_init;
 
   int               cs_convert_init;
+  int               is_direct_rendering_disabled;
+
   AVPaletteControl  palette_control;
+  
+  xine_list_t       *dr1_frames;
 };
 
 
@@ -146,11 +151,16 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
   avcodec_align_dimensions(context, &width, &height);
 
   if( this->context->pix_fmt != PIX_FMT_YUV420P ) {
-    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
-            _("ffmpeg_video_dec: unsupported frame format, DR1 disabled.\n"));
+    if (!this->is_direct_rendering_disabled) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+              _("ffmpeg_video_dec: unsupported frame format, DR1 disabled.\n"));
+      this->is_direct_rendering_disabled = 1;
+    }
 
-    this->context->get_buffer = avcodec_default_get_buffer;
-    this->context->release_buffer = avcodec_default_release_buffer;
+    /* FIXME: why should i have to do that ? */
+    av_frame->data[0]= NULL;
+    av_frame->data[1]= NULL;
+    av_frame->data[2]= NULL;
     return avcodec_default_get_buffer(context, av_frame);
   }
   
@@ -159,11 +169,15 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
       this->crop_right = width - this->bih.biWidth;
       this->crop_bottom = height - this->bih.biHeight;
     } else {
-      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
-              _("ffmpeg_video_dec: unsupported frame dimensions, DR1 disabled.\n"));
-
-      this->context->get_buffer = avcodec_default_get_buffer;
-      this->context->release_buffer = avcodec_default_release_buffer;
+      if (!this->is_direct_rendering_disabled) {
+        xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+                _("ffmpeg_video_dec: unsupported frame dimensions, DR1 disabled.\n"));
+        this->is_direct_rendering_disabled = 1;
+      }
+      /* FIXME: why should i have to do that ? */
+      av_frame->data[0]= NULL;
+      av_frame->data[1]= NULL;
+      av_frame->data[2]= NULL;
       return avcodec_default_get_buffer(context, av_frame);
     }
   }
@@ -192,22 +206,33 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
 
   av_frame->type= FF_BUFFER_TYPE_USER;
 
+  xine_list_push_back(this->dr1_frames, av_frame);
+
   return 0;
 }
 
 static void release_buffer(struct AVCodecContext *context, AVFrame *av_frame){
-  vo_frame_t *img = (vo_frame_t *)av_frame->opaque;
+  ff_video_decoder_t *this = (ff_video_decoder_t *)context->opaque;
+
+  if (av_frame->type == FF_BUFFER_TYPE_USER) {
+    vo_frame_t *img = (vo_frame_t *)av_frame->opaque;
+    xine_list_iterator_t it;
     
-  assert(av_frame->type == FF_BUFFER_TYPE_USER);
-  assert(av_frame->opaque);  
-  
+    assert(av_frame->opaque);  
+    img->free(img);
+    
+    it = xine_list_find(this->dr1_frames, av_frame);
+    assert(it);
+    if( it != NULL )
+      xine_list_remove(this->dr1_frames, it);
+  } else {
+    avcodec_default_release_buffer(context, av_frame);
+  }
+
+  av_frame->opaque = NULL;
   av_frame->data[0]= NULL;
   av_frame->data[1]= NULL;
   av_frame->data[2]= NULL;
-
-  img->free(img);
-
-  av_frame->opaque = NULL;
 }
 #endif
 
@@ -217,13 +242,14 @@ static const ff_codec_t ff_video_lookup[] = {
   {BUF_VIDEO_MSMPEG4_V3,  CODEC_ID_MSMPEG4V3, "Microsoft MPEG-4 v3 (ffmpeg)"},
   {BUF_VIDEO_WMV7,        CODEC_ID_WMV1,      "MS Windows Media Video 7 (ffmpeg)"},
   {BUF_VIDEO_WMV8,        CODEC_ID_WMV2,      "MS Windows Media Video 8 (ffmpeg)"},
+  {BUF_VIDEO_WMV9,        CODEC_ID_WMV3,      "MS Windows Media Video 9 (ffmpeg)"},
   {BUF_VIDEO_MPEG4,       CODEC_ID_MPEG4,     "ISO MPEG-4 (ffmpeg)"},
   {BUF_VIDEO_XVID,        CODEC_ID_MPEG4,     "ISO MPEG-4 (XviD, ffmpeg)"},
   {BUF_VIDEO_DIVX5,       CODEC_ID_MPEG4,     "ISO MPEG-4 (DivX5, ffmpeg)"},
   {BUF_VIDEO_3IVX,        CODEC_ID_MPEG4,     "ISO MPEG-4 (3ivx, ffmpeg)"},
   {BUF_VIDEO_JPEG,        CODEC_ID_MJPEG,     "Motion JPEG (ffmpeg)"},
   {BUF_VIDEO_MJPEG,       CODEC_ID_MJPEG,     "Motion JPEG (ffmpeg)"},
-  {BUF_VIDEO_MJPEG_B,     CODEC_ID_MJPEGB,    "Motion JPEG B (ffmpeg"},
+  {BUF_VIDEO_MJPEG_B,     CODEC_ID_MJPEGB,    "Motion JPEG B (ffmpeg)"},
   {BUF_VIDEO_I263,        CODEC_ID_H263I,     "ITU H.263 (ffmpeg)"},
   {BUF_VIDEO_H263,        CODEC_ID_H263,      "H.263 (ffmpeg)"},
   {BUF_VIDEO_RV10,        CODEC_ID_RV10,      "Real Video 1.0 (ffmpeg)"},
@@ -235,6 +261,9 @@ static const ff_codec_t ff_video_lookup[] = {
   {BUF_VIDEO_DV,          CODEC_ID_DVVIDEO,   "DV (ffmpeg)"},
   {BUF_VIDEO_HUFFYUV,     CODEC_ID_HUFFYUV,   "HuffYUV (ffmpeg)"},
   {BUF_VIDEO_VP31,        CODEC_ID_VP3,       "On2 VP3.1 (ffmpeg)"},
+  {BUF_VIDEO_VP5,         CODEC_ID_VP5,       "On2 VP5 (ffmpeg)"},
+  {BUF_VIDEO_VP6,         CODEC_ID_VP6,       "On2 VP6 (ffmpeg)"},
+  {BUF_VIDEO_VP6F,        CODEC_ID_VP6F,      "On2 VP6 (ffmpeg)"},
   {BUF_VIDEO_4XM,         CODEC_ID_4XM,       "4X Video (ffmpeg)"},
   {BUF_VIDEO_CINEPAK,     CODEC_ID_CINEPAK,   "Cinepak (ffmpeg)"},
   {BUF_VIDEO_MSVC,        CODEC_ID_MSVIDEO1,  "Microsoft Video 1 (ffmpeg)"},
@@ -271,7 +300,17 @@ static const ff_codec_t ff_video_lookup[] = {
   {BUF_VIDEO_XL,          CODEC_ID_VIXL,       "Miro/Pinnacle VideoXL (ffmpeg)"},
   {BUF_VIDEO_RT21,        CODEC_ID_INDEO2,     "Indeo/RealTime 2 (ffmpeg)"},
   {BUF_VIDEO_FPS1,        CODEC_ID_FRAPS,      "Fraps (ffmpeg)"},
-  {BUF_VIDEO_MPEG,        CODEC_ID_MPEG1VIDEO, "MPEG 1/2 (ffmpeg)"} };
+  {BUF_VIDEO_MPEG,        CODEC_ID_MPEG1VIDEO, "MPEG 1/2 (ffmpeg)"},
+  {BUF_VIDEO_CSCD,        CODEC_ID_CSCD,       "CamStudio (ffmpeg)"},
+  {BUF_VIDEO_AVS,         CODEC_ID_AVS,        "AVS (ffmpeg)"},
+  {BUF_VIDEO_ALGMM,       CODEC_ID_MMVIDEO,    "American Laser Games MM (ffmpeg)"},
+  {BUF_VIDEO_ZMBV,        CODEC_ID_ZMBV,       "Zip Motion Blocks Video (ffmpeg)"},
+  {BUF_VIDEO_SMACKER,     CODEC_ID_SMACKVIDEO, "Smacker (ffmpeg)"},
+  {BUF_VIDEO_NUV,         CODEC_ID_NUV,        "NuppelVideo (ffmpeg)"},
+  {BUF_VIDEO_KMVC,        CODEC_ID_KMVC,       "Karl Morton's Video Codec (ffmpeg)"},
+  {BUF_VIDEO_FLASHSV,     CODEC_ID_FLASHSV,    "Flash Screen Video (ffmpeg)"},
+  {BUF_VIDEO_CAVS,        CODEC_ID_CAVS,       "Chinese AVS (ffmpeg)"},
+};
 
 
 static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type) {
@@ -352,7 +391,7 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
   /* enable direct rendering by default */
   this->output_format = XINE_IMGFMT_YV12;
 #ifdef ENABLE_DIRECT_RENDERING
-  if( this->codec->capabilities & CODEC_CAP_DR1 ) {
+  if( this->codec->capabilities & CODEC_CAP_DR1 && this->codec->id != CODEC_ID_H264 ) {
     this->context->get_buffer = get_buffer;
     this->context->release_buffer = release_buffer;
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
@@ -391,7 +430,7 @@ static void pp_change_quality (ff_video_decoder_t *this) {
   this->pp_quality = this->class->pp_quality;
 
   if(this->pp_available && this->pp_quality) {
-    if(!this->pp_context)
+    if(!this->pp_context && this->context)
       this->pp_context = pp_get_context(this->context->width, this->context->height,
                                         this->pp_flags);
     if(this->pp_mode)
@@ -777,7 +816,7 @@ static void ff_check_bufsize (ff_video_decoder_t *this, int size) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
 	    _("ffmpeg_video_dec: increasing buffer to %d to avoid overflow.\n"), 
 	    this->bufsize);
-    this->buf = realloc(this->buf, this->bufsize);
+    this->buf = realloc(this->buf, this->bufsize + FF_INPUT_BUFFER_PADDING_SIZE );
   }
 }
 
@@ -802,7 +841,7 @@ static void ff_handle_header_buffer (ff_video_decoder_t *this, buf_element_t *bu
   lprintf ("header buffer\n");
 
   /* accumulate data */
-  ff_check_bufsize(this, this->size + buf->size + FF_INPUT_BUFFER_PADDING_SIZE);
+  ff_check_bufsize(this, this->size + buf->size);
   xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
   this->size += buf->size;
 
@@ -820,7 +859,7 @@ static void ff_handle_header_buffer (ff_video_decoder_t *this, buf_element_t *bu
       memcpy ( &this->bih, this->buf, sizeof(xine_bmiheader) );
 
       if (this->bih.biSize > sizeof(xine_bmiheader)) {
-        this->context->extradata_size = this->bih.biSize - sizeof(xine_bmiheader);
+      this->context->extradata_size = this->bih.biSize - sizeof(xine_bmiheader);
         this->context->extradata = malloc(this->context->extradata_size + 
                                           FF_INPUT_BUFFER_PADDING_SIZE);
         memcpy(this->context->extradata, this->buf + sizeof(xine_bmiheader),
@@ -1078,7 +1117,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
       lprintf("no memcpy needed to accumulate data\n");
     } else {
       /* copy data into our internal buffer */
-      ff_check_bufsize(this, this->size + buf->size + FF_INPUT_BUFFER_PADDING_SIZE);
+      ff_check_bufsize(this, this->size + buf->size);
       chunk_buf = this->buf; /* ff_check_bufsize might realloc this->buf */
 
       xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
@@ -1098,7 +1137,13 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
     int         codec_type = buf->type & 0xFFFF0000;
 
     /* pad input data */
-    chunk_buf[this->size] = 0;
+    /* note: bitstream, alt bitstream reader or something will cause
+     * severe mpeg4 artifacts if padding is less than 32 bits.
+     */
+    chunk_buf[this->size+0] = 0;
+    chunk_buf[this->size+1] = 0;
+    chunk_buf[this->size+2] = 0;
+    chunk_buf[this->size+3] = 0;
 
     while (this->size > 0) {
       
@@ -1126,7 +1171,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
           this->size -= len;
 
           if (this->size > 0) {
-            ff_check_bufsize(this, this->size + FF_INPUT_BUFFER_PADDING_SIZE);
+            ff_check_bufsize(this, this->size);
             memmove (this->buf, &chunk_buf[offset], this->size);
             chunk_buf = this->buf;
           }
@@ -1232,7 +1277,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         img->crop_bottom = this->crop_bottom;
         
         this->skipframes = img->draw(img, this->stream);
-
+        
         if(free_img)
           img->free(img);
       }
@@ -1336,12 +1381,23 @@ static void ff_dispose (video_decoder_t *this_gen) {
   ff_video_decoder_t *this = (ff_video_decoder_t *) this_gen;
 
   lprintf ("ff_dispose\n");
-
+  
   if (this->decoder_ok) {
+    xine_list_iterator_t it;
+    AVFrame *av_frame;
+        
     pthread_mutex_lock(&ffmpeg_lock);
     avcodec_close (this->context);
     pthread_mutex_unlock(&ffmpeg_lock);
-
+    
+    /* frame garbage collector here - workaround for buggy ffmpeg codecs that
+     * don't release their DR1 frames */
+    while( (it = xine_list_front(this->dr1_frames)) != NULL )
+    {
+      av_frame = (AVFrame *)xine_list_get_value(this->dr1_frames, it);
+      release_buffer(this->context, av_frame);
+    }
+    
     this->stream->video_out->close(this->stream->video_out, this->stream);
     this->decoder_ok = 0;
   }
@@ -1370,6 +1426,8 @@ static void ff_dispose (video_decoder_t *this_gen) {
     
   if(this->pp_mode)
     pp_free_mode(this->pp_mode);
+    
+  xine_list_delete(this->dr1_frames);
   
   free (this_gen);
 }
@@ -1409,6 +1467,8 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
   this->pp_context        = NULL;
   this->pp_mode           = NULL;
   
+  this->dr1_frames        = xine_list_new();
+  
   mpeg_parser_init(&this->mpeg_parser);
 
   return &this->video_decoder;
@@ -1459,64 +1519,224 @@ void *init_video_plugin (xine_t *xine, void *data) {
 }
 
 static uint32_t supported_video_types[] = { 
-  BUF_VIDEO_MSMPEG4_V1, 
+  #ifdef CONFIG_MSMPEG4V1_DECODER
+  BUF_VIDEO_MSMPEG4_V1,
+  #endif
+  #ifdef CONFIG_MSMPEG4V2_DECODER
   BUF_VIDEO_MSMPEG4_V2,
-  BUF_VIDEO_MSMPEG4_V3, 
-  BUF_VIDEO_WMV7, 
+  #endif
+  #ifdef CONFIG_MSMPEG4V3_DECODER
+  BUF_VIDEO_MSMPEG4_V3,
+  #endif
+  #ifdef CONFIG_WMV1_DECODER
+  BUF_VIDEO_WMV7,
+  #endif
+  #ifdef CONFIG_WMV2_DECODER
+  BUF_VIDEO_WMV8,
+  #endif
+  #ifdef CONFIG_WMV3_DECODER
+  BUF_VIDEO_WMV9,
+  #endif
+  #ifdef CONFIG_MPEG4_DECODER
   BUF_VIDEO_MPEG4,
-  BUF_VIDEO_XVID, 
-  BUF_VIDEO_DIVX5, 
+  #endif
+  #ifdef CONFIG_MPEG4_DECODER
+  BUF_VIDEO_XVID,
+  #endif
+  #ifdef CONFIG_MPEG4_DECODER
+  BUF_VIDEO_DIVX5,
+  #endif
+  #ifdef CONFIG_MPEG4_DECODER
   BUF_VIDEO_3IVX,
+  #endif
+  #ifdef CONFIG_MJPEG_DECODER
+  BUF_VIDEO_JPEG,
+  #endif
+  #ifdef CONFIG_MJPEG_DECODER
   BUF_VIDEO_MJPEG,
+  #endif
+  #ifdef CONFIG_MJPEGB_DECODER
   BUF_VIDEO_MJPEG_B,
+  #endif
+  #ifdef CONFIG_H263I_DECODER
+  BUF_VIDEO_I263,
+  #endif
+  #ifdef CONFIG_H263_DECODER
   BUF_VIDEO_H263,
+  #endif
+  #ifdef CONFIG_RV10_DECODER
   BUF_VIDEO_RV10,
+  #endif
+  #ifdef CONFIG_RV20_DECODER
   BUF_VIDEO_RV20,
+  #endif
+  #ifdef CONFIG_INDEO3_DECODER
   BUF_VIDEO_IV31,
+  #endif
+  #ifdef CONFIG_INDEO3_DECODER
   BUF_VIDEO_IV32,
+  #endif
+  #ifdef CONFIG_SVQ1_DECODER
   BUF_VIDEO_SORENSON_V1,
+  #endif
+  #ifdef CONFIG_SVQ3_DECODER
   BUF_VIDEO_SORENSON_V3,
-  BUF_VIDEO_JPEG, 
-  BUF_VIDEO_MPEG, 
+  #endif
+  #ifdef CONFIG_DVVIDEO_DECODER
   BUF_VIDEO_DV,
+  #endif
+  #ifdef CONFIG_HUFFYUV_DECODER
   BUF_VIDEO_HUFFYUV,
+  #endif
+  #ifdef CONFIG_VP3_DECODER
   BUF_VIDEO_VP31,
+  #endif
+  #ifdef CONFIG_VP5_DECODER
+  BUF_VIDEO_VP5,
+  #endif
+  #ifdef CONFIG_VP6_DECODER
+  BUF_VIDEO_VP6,
+  BUF_VIDEO_VP6F,
+  #endif
+  #ifdef CONFIG_4XM_DECODER
   BUF_VIDEO_4XM,
+  #endif
+  #ifdef CONFIG_CINEPAK_DECODER
   BUF_VIDEO_CINEPAK,
+  #endif
+  #ifdef CONFIG_MSVIDEO1_DECODER
   BUF_VIDEO_MSVC,
+  #endif
+  #ifdef CONFIG_MSRLE_DECODER
   BUF_VIDEO_MSRLE,
+  #endif
+  #ifdef CONFIG_RPZA_DECODER
   BUF_VIDEO_RPZA,
+  #endif
+  #ifdef CONFIG_CYUV_DECODER
   BUF_VIDEO_CYUV,
+  #endif
+  #ifdef CONFIG_ROQ_DECODER
   BUF_VIDEO_ROQ,
+  #endif
+  #ifdef CONFIG_IDCIN_DECODER
   BUF_VIDEO_IDCIN,
+  #endif
+  #ifdef CONFIG_XAN_WC3_DECODER
   BUF_VIDEO_WC3,
+  #endif
+  #ifdef CONFIG_WS_VQA_DECODER
   BUF_VIDEO_VQA,
+  #endif
+  #ifdef CONFIG_INTERPLAY_VIDEO_DECODER
   BUF_VIDEO_INTERPLAY,
+  #endif
+  #ifdef CONFIG_FLIC_DECODER
   BUF_VIDEO_FLI,
+  #endif
+  #ifdef CONFIG_8BPS_DECODER
   BUF_VIDEO_8BPS,
+  #endif
+  #ifdef CONFIG_SMC_DECODER
   BUF_VIDEO_SMC,
-  BUF_VIDEO_VMD,
+  #endif
+  #ifdef CONFIG_TRUEMOTION1_DECODER
   BUF_VIDEO_DUCKTM1,
+  #endif
+  #ifdef CONFIG_TRUEMOTION2_DECODER
   BUF_VIDEO_DUCKTM2,
+  #endif
+  #ifdef CONFIG_VMDVIDEO_DECODER
+  BUF_VIDEO_VMD,
+  #endif
+  #ifdef CONFIG_ZLIB_DECODER
   BUF_VIDEO_ZLIB,
+  #endif
+  #ifdef CONFIG_MSZH_DECODER
   BUF_VIDEO_MSZH,
+  #endif
+  #ifdef CONFIG_ASV1_DECODER
   BUF_VIDEO_ASV1,
+  #endif
+  #ifdef CONFIG_ASV2_DECODER
   BUF_VIDEO_ASV2,
+  #endif
+  #ifdef CONFIG_VCR1_DECODER
   BUF_VIDEO_ATIVCR1,
+  #endif
+  #ifdef CONFIG_FLV_DECODER
   BUF_VIDEO_FLV1,
+  #endif
+  #ifdef CONFIG_QTRLE_DECODER
   BUF_VIDEO_QTRLE,
+  #endif
+  #ifdef CONFIG_H264_DECODER
   BUF_VIDEO_H264,
+  #endif
+  #ifdef CONFIG_H261_DECODER
   BUF_VIDEO_H261,
+  #endif
+  #ifdef CONFIG_AASC_DECODER
   BUF_VIDEO_AASC,
+  #endif
+  #ifdef CONFIG_LOCO_DECODER
   BUF_VIDEO_LOCO,
+  #endif
+  #ifdef CONFIG_QDRAW_DECODER
   BUF_VIDEO_QDRW,
+  #endif
+  #ifdef CONFIG_QPEG_DECODER
   BUF_VIDEO_QPEG,
+  #endif
+  #ifdef CONFIG_TSCC_DECODER
   BUF_VIDEO_TSCC,
+  #endif
+  #ifdef CONFIG_ULTI_DECODER
   BUF_VIDEO_ULTI,
+  #endif
+  #ifdef CONFIG_WNV1_DECODER
   BUF_VIDEO_WNV1,
+  #endif
+  #ifdef CONFIG_VIXL_DECODER
   BUF_VIDEO_XL,
+  #endif
+  #ifdef CONFIG_INDEO2_DECODER
   BUF_VIDEO_RT21,
+  #endif
+  #ifdef CONFIG_FRAPS_DECODER
   BUF_VIDEO_FPS1,
+  #endif
+  #ifdef CONFIG_MPEG1VIDEO_DECODER
+  BUF_VIDEO_MPEG,
+  #endif
+  #ifdef CONFIG_CSCD_DECODER
+  BUF_VIDEO_CSCD,
+  #endif
+  #ifdef CONFIG_AVS_DECODER
+  BUF_VIDEO_AVS,
+  #endif
+  #ifdef CONFIG_MMVIDEO_DECODER
+  BUF_VIDEO_ALGMM,
+  #endif
+  #ifdef CONFIG_ZMBV_DECODER
+  BUF_VIDEO_ZMBV,
+  #endif
+  #ifdef CONFIG_SMACKVIDEO_DECODER
+  BUF_VIDEO_SMACKER,
+  #endif
+  #ifdef CONFIG_NUV_DECODER
+  BUF_VIDEO_NUV,
+  #endif
+  #ifdef CONFIG_KMVC_DECODER
+  BUF_VIDEO_KMVC,
+  #endif
+  #ifdef CONFIG_FLASHSV_DECODER
+  BUF_VIDEO_FLASHSV,
+  #endif
+  #ifdef CONFIG_CAVS_DECODER
+  BUF_VIDEO_CAVS,
+  #endif
+
   0 
 };
 
@@ -1525,6 +1745,11 @@ static uint32_t wmv8_video_types[] = {
   0 
 };
 
+static uint32_t wmv9_video_types[] = { 
+  BUF_VIDEO_WMV9,
+  0 
+};
+
 decoder_info_t dec_info_ffmpeg_video = {
   supported_video_types,   /* supported types */
   6                        /* priority        */
@@ -1534,3 +1759,8 @@ decoder_info_t dec_info_ffmpeg_wmv8 = {
   wmv8_video_types,        /* supported types */
   0                        /* priority        */
 };
+
+decoder_info_t dec_info_ffmpeg_wmv9 = {
+  wmv9_video_types,        /* supported types */
+  0                        /* priority        */
+};
