@@ -29,6 +29,7 @@
 #include <string.h>	/* memcpy/memset, try to remove */
 #include <stdlib.h>
 #include <inttypes.h>
+#include <math.h>
 
 #define LOG_MODULE "decode"
 #define LOG_VERBOSE
@@ -69,10 +70,10 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec,
 
     if( !mpeg2dec->chunk_buffer )
       mpeg2dec->chunk_buffer = xine_xmalloc_aligned (16, BUFFER_SIZE + 4, 
-						     (void**)&mpeg2dec->chunk_base);
+						     &mpeg2dec->chunk_base);
     if( !mpeg2dec->picture )
       mpeg2dec->picture = xine_xmalloc_aligned (16, sizeof (picture_t),
-						(void**)&mpeg2dec->picture_base);
+						&mpeg2dec->picture_base);
 
     mpeg2dec->shift = 0xffffff00;
     mpeg2dec->new_sequence = 0;
@@ -104,40 +105,22 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec,
 
 static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
 {
-  switch (mpeg2dec->picture->frame_rate_code) {
-  case 1: /* 23.976 fps */
-    frame->duration      = 3754;  /* actually it's 3753.75 */
-    break;
-  case 2: /* 24 fps */
-    frame->duration      = 3750;
-    break;
-  case 3: /* 25 fps */
-    frame->duration      = 3600;
-    break;
-  case 4: /* 29.97 fps */
-    frame->duration      = 3003;
-    break;
-  case 5: /* 30 fps */
-    frame->duration      = 3000;
-    break;
-  case 6: /* 50 fps */
-    frame->duration      = 1800;
-    break;
-  case 7: /* 59.94 fps */
-    frame->duration      = 1502;  /* actually it's 1501.5 */
-    break;
-  case 8: /* 60 fps */
-    frame->duration      = 1500;
-    break;
-  default:
-       /* printf ("invalid/unknown frame rate code : %d \n",
-               frame->frame_rate_code); */
-    frame->duration      = 0;
-  }
+  static const double durations[] = {
+    0,		/* invalid */
+    3753.75,	/* 23.976 fps */
+    3750,	/* 24 fps */
+    3600,	/* 25 fps */
+    3003,	/* 29.97 fps */
+    3000,	/* 30 fps */
+    1800,	/* 50 fps */
+    1501.5,	/* 59.94 fps */
+    1500,	/* 60 fps */
+  };
+  double duration = ((unsigned) mpeg2dec->picture->frame_rate_code > 8u)
+		    ? 0 : durations[mpeg2dec->picture->frame_rate_code];
   
-  frame->duration = frame->duration *
-    ((mpeg2dec->picture->frame_rate_ext_n + 1) /
-     (mpeg2dec->picture->frame_rate_ext_d + 1));
+  duration = duration * (mpeg2dec->picture->frame_rate_ext_n + 1.0) /
+			(mpeg2dec->picture->frame_rate_ext_d + 1.0);
  
   /* this should be used to detect any special rff pattern */
   mpeg2dec->rff_pattern = mpeg2dec->rff_pattern << 1;
@@ -147,7 +130,7 @@ static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
       (mpeg2dec->rff_pattern & 0xff) == 0x55) &&
       !mpeg2dec->picture->progressive_sequence ) {
     /* special case for ntsc 3:2 pulldown */
-    frame->duration += frame->duration/4;
+    duration *= 5.0 / 4.0;
   }
   else
   {  
@@ -156,14 +139,16 @@ static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
            frame->progressive_frame ) {
         /* decoder should output 3 fields, so adjust duration to
            count on this extra field time */
-        frame->duration += frame->duration/2;     
+        duration *= 3.0 / 2.0;
       } else if( mpeg2dec->picture->progressive_sequence ) {
         /* for progressive sequences the output should repeat the
            frame 1 or 2 times depending on top_field_first flag. */
-        frame->duration *= (frame->top_field_first)?3:2;
+        duration *= (frame->top_field_first) ? 3 : 2;
       }
     }
   }
+
+  frame->duration = (int) ceil (duration);
   _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, frame->duration);
   /*printf("mpeg2dec: rff=%u\n",frame->repeat_first_field);*/
 } 
@@ -415,6 +400,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	}
 
 	if (mpeg2dec->is_sequence_needed 
+	    || (picture->aspect_ratio_information != picture->saved_aspect_ratio)
 	    || (picture->frame_width != picture->coded_picture_width)
 	    || (picture->frame_height != picture->coded_picture_height)) {
 	    xine_event_t event;
@@ -451,6 +437,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 
 	    picture->frame_width = picture->coded_picture_width;
 	    picture->frame_height = picture->coded_picture_height;
+	    picture->saved_aspect_ratio = picture->aspect_ratio_information;
 	}
 	break;
 
@@ -508,7 +495,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 						     mpeg2dec->frame_format,
 						     flags);
 		    libmpeg2_accel_new_frame( &mpeg2dec->accel, mpeg2dec->frame_format,
-					      picture, ratio);
+					      picture, ratio, flags);
 		} else {
                     ratio = get_aspect_ratio(mpeg2dec);
 		    picture->current_frame =
@@ -520,7 +507,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 						     flags);
 
 		    libmpeg2_accel_new_frame( &mpeg2dec->accel, mpeg2dec->frame_format,
-					      picture, ratio);
+					      picture, ratio, flags);
 
 		    if (picture->forward_reference_frame &&
 		        picture->forward_reference_frame != picture->backward_reference_frame)
@@ -573,9 +560,11 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	  printf("slice target %08x past %08x future %08x\n",picture->current_frame,picture->forward_reference_frame,picture->backward_reference_frame);
 	  fflush(stdout);
 #endif
-	  libmpeg2_accel_slice(&mpeg2dec->accel, mpeg2dec->frame_format, picture, code, buffer, 
-			       mpeg2dec->chunk_size, mpeg2dec->chunk_buffer);
-	  if( picture->v_offset > picture->limit_y ) { 
+	  libmpeg2_accel_slice(&mpeg2dec->accel, picture, code, buffer, mpeg2dec->chunk_size, 
+			       mpeg2dec->chunk_buffer);
+
+	  if( picture->v_offset > picture->limit_y || 
+	      picture->v_offset + 16 > picture->display_height ) { 
 	    picture->current_frame->bad_frame = 0;
 	  }
 	}
@@ -664,6 +653,7 @@ void mpeg2_discontinuity (mpeg2dec_t * mpeg2dec) {
   if( !picture )
     return;
   
+  mpeg2dec->in_slice = 0;
   mpeg2dec->pts = 0;  
   if ( picture->current_frame )
     picture->current_frame->pts = 0;
@@ -926,4 +916,11 @@ static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer)
       mpeg2dec->cc_dec->decode_data(mpeg2dec->cc_dec, &buf);
     }
   }
+  /* check Active Format Description ETSI TS 101 154 V1.5.1 */
+  else if (buffer[0] == 0x44 && buffer[1] == 0x54 && buffer[2] == 0x47 && buffer[3] == 0x31)
+  {
+    int afd = (buffer[4] & 0x40) ? (buffer[5] & 0x0f) : -1;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_AFD, afd);
+    
+  }
 }
