@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_decoder.c,v 1.43 2004/12/16 13:59:10 mroi Exp $
+ * $Id: audio_decoder.c,v 1.50 2006/12/19 19:10:51 dsalt Exp $
  *
  * thin layer to use real binary-only codecs in xine
  *
@@ -164,14 +164,18 @@ static inline int is_x86_64_object(const char *filename)
 }
 #endif
 
-static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
+static int load_syms_linux (realdec_decoder_t *this, char *codec_name,
+			    const char *alt_codec_name) {
 
   cfg_entry_t* entry = this->stream->xine->config->lookup_entry(
 			 this->stream->xine->config, "decoder.external.real_codecs_path");
   char path[1024];
+  struct stat sb;
 
   snprintf (path, sizeof(path), "%s/%s", entry->str_value, codec_name);
-
+  if (stat(path, &sb))
+    snprintf (path, sizeof(path), "%s/%s", entry->str_value, alt_codec_name);
+    
 #ifdef __x86_64__
   /* check whether it's a real x86-64 library */
   if (!is_x86_64_object(path))
@@ -212,7 +216,7 @@ static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
 
     char path[1024];
 
-    snprintf(path, sizeof(path) - 1, "DT_Codecs=%s", entry->str_value);
+    snprintf(path, sizeof(path) - 2, "DT_Codecs=%s", entry->str_value);
     if (path[strlen(path)-1]!='/'){
       path[strlen(path)+1]=0;
       path[strlen(path)]='/';
@@ -293,32 +297,32 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
   switch (buf->type) {
   case BUF_AUDIO_COOK:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Cook");
-    if (!load_syms_linux (this, "cook.so.6.0"))
+    if (!load_syms_linux (this, "cook.so", "cook.so.6.0"))
       return 0;
     break;
     
   case BUF_AUDIO_ATRK:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Atrac");
-    if (!load_syms_linux (this, "atrc.so.6.0"))
+    if (!load_syms_linux (this, "atrc.so", "atrc.so.6.0"))
       return 0;
     this->block_align = 384;
     break;
 
   case BUF_AUDIO_14_4:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Real 14.4");
-    if (!load_syms_linux (this, "14_4.so.6.0"))
+    if (!load_syms_linux (this, "14_4.so", "14_4.so.6.0"))
       return 0;
     break;
 
   case BUF_AUDIO_28_8:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Real 28.8");
-    if (!load_syms_linux (this, "28_8.so.6.0"))
+    if (!load_syms_linux (this, "28_8.so", "28_8.so.6.0"))
       return 0;
     break;
 
   case BUF_AUDIO_SIPRO:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Sipro");
-    if (!load_syms_linux (this, "sipr.so.6.0"))
+    if (!load_syms_linux (this, "sipr.so", "sipr.so.6.0"))
       return 0;
     /* this->block_align = 19; */
     break;
@@ -677,6 +681,10 @@ static void dispose_class (audio_decoder_class_t *this) {
 /*
  * some fake functions to make real codecs happy 
  */
+void *__builtin_vec_new(unsigned long size) EXPORTED;
+void __builtin_vec_delete(void *mem) EXPORTED;
+void __pure_virtual(void) EXPORTED;
+
 void *__builtin_vec_new(unsigned long size) {
   return malloc(size);
 }
@@ -719,15 +727,22 @@ static void *init_class (xine_t *xine, void *data) {
     default_real_codec_path = "/opt/RealPlayer8/Codecs";
   if (!stat ("/usr/lib/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib/RealPlayer9/users/Real/Codecs";
+  if (!stat ("/usr/lib/RealPlayer10/codecs/drvc.so", &s)) 
+    default_real_codec_path = "/usr/lib/RealPlayer10/codecs";
   if (!stat ("/usr/lib64/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib64/RealPlayer8/Codecs";
   if (!stat ("/usr/lib64/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib64/RealPlayer9/users/Real/Codecs";
+  if (!stat ("/usr/lib64/RealPlayer10/codecs/drvc.so", &s)) 
+    default_real_codec_path = "/usr/lib64/RealPlayer10/codecs";
+  if (!stat ("/usr/lib/codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/lib/codecs";
   if (!stat ("/usr/lib/win32/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib/win32";
   
-  real_codec_path = config->register_string (config, "decoder.external.real_codecs_path", 
+  real_codec_path = config->register_filename (config, "decoder.external.real_codecs_path", 
 					     default_real_codec_path,
+					     XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
 					     _("path to RealPlayer codecs"),
 					     _("If you have RealPlayer installed, specify the path "
 					       "to its codec directory here. You can easily find "
@@ -751,12 +766,12 @@ static uint32_t audio_types[] = {
   BUF_AUDIO_COOK, BUF_AUDIO_ATRK, /* BUF_AUDIO_14_4, BUF_AUDIO_28_8, */ BUF_AUDIO_SIPRO, 0
  };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   5                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 15, "realadec", XINE_VERSION_CODE, &dec_info_audio, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
