@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.79 2005/10/27 14:58:56 mroi Exp $
+ * $Id: xine_decoder.c,v 1.84 2006/12/19 19:10:51 dsalt Exp $
  *
  * thin layer to use real binary-only codecs in xine
  *
@@ -153,13 +153,17 @@ static inline int is_x86_64_object(const char *filename)
  * real codec loader
  */
 
-static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
+static int load_syms_linux (realdec_decoder_t *this, char *codec_name,
+			    const char *alt_codec_name) {
 
   cfg_entry_t* entry = this->stream->xine->config->lookup_entry(
 			 this->stream->xine->config, "decoder.external.real_codecs_path");
   char path[1024];
+  struct stat sb;
 
   snprintf (path, sizeof(path), "%s/%s", entry->str_value, codec_name);
+  if (stat(path, &sb))
+    snprintf (path, sizeof(path), "%s/%s", entry->str_value, alt_codec_name);
 
 #ifdef __x86_64__
   /* check whether it's a real x86-64 library */
@@ -184,6 +188,19 @@ static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
   this->rvyuv_init           = dlsym (this->rv_handle, "RV20toYUV420Init");
   this->rvyuv_transform      = dlsym (this->rv_handle, "RV20toYUV420Transform");
   
+  if (this->rvyuv_custom_message &&
+      this->rvyuv_free &&
+      this->rvyuv_hive_message &&
+      this->rvyuv_init &&
+      this->rvyuv_transform) 
+    return 1;
+
+  this->rvyuv_custom_message = dlsym (this->rv_handle, "RV40toYUV420CustomMessage");
+  this->rvyuv_free           = dlsym (this->rv_handle, "RV40toYUV420Free");
+  this->rvyuv_hive_message   = dlsym (this->rv_handle, "RV40toYUV420HiveMessage");
+  this->rvyuv_init           = dlsym (this->rv_handle, "RV40toYUV420Init");
+  this->rvyuv_transform      = dlsym (this->rv_handle, "RV40toYUV420Transform");
+  
   if (this->rvyuv_custom_message &&
       this->rvyuv_free &&
       this->rvyuv_hive_message &&
@@ -206,17 +223,17 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
   switch (buf->type) {
   case BUF_VIDEO_RV20:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "Real Video 2.0");
-    if (!load_syms_linux (this, "drv2.so.6.0"))
+    if (!load_syms_linux (this, "drv2.so", "drv2.so.6.0"))
       return 0;
     break;
   case BUF_VIDEO_RV30:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "Real Video 3.0");
-    if (!load_syms_linux (this, "drv3.so.6.0"))
+    if (!load_syms_linux (this, "drvc.so", "drv3.so.6.0"))
       return 0;
     break;
   case BUF_VIDEO_RV40:
     _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "Real Video 4.0");
-    if (!load_syms_linux(this, "drv4.so.6.0"))
+    if (!load_syms_linux(this, "drvc.so", "drv4.so.6.0"))
       return 0;
     break;
   default:
@@ -560,6 +577,10 @@ static void dispose_class (video_decoder_class_t *this) {
 /*
  * some fake functions to make real codecs happy 
  */
+void *__builtin_vec_new(uint32_t size) EXPORTED;
+void __builtin_vec_delete(void *mem) EXPORTED;
+void __pure_virtual(void) EXPORTED;
+
 void *__builtin_vec_new(uint32_t size) {
   return malloc(size);
 }
@@ -599,17 +620,22 @@ static void *init_class (xine_t *xine, void *data) {
     default_real_codec_path = "/opt/RealPlayer8/Codecs";
   if (!stat ("/usr/lib/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib/RealPlayer9/users/Real/Codecs";
+  if (!stat ("/usr/lib/RealPlayer10/codecs/drvc.so", &s)) 
+    default_real_codec_path = "/usr/lib/RealPlayer10/codecs";
   if (!stat ("/usr/lib64/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib64/RealPlayer8/Codecs";
   if (!stat ("/usr/lib64/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib64/RealPlayer9/users/Real/Codecs";
+  if (!stat ("/usr/lib64/RealPlayer10/codecs/drvc.so", &s)) 
+    default_real_codec_path = "/usr/lib64/RealPlayer10/codecs";
   if (!stat ("/usr/lib/codecs/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib/codecs";
   if (!stat ("/usr/lib/win32/drv3.so.6.0", &s)) 
     default_real_codec_path = "/usr/lib/win32";
   
-  real_codec_path = config->register_string (config, "decoder.external.real_codecs_path", 
+  real_codec_path = config->register_filename (config, "decoder.external.real_codecs_path", 
 					     default_real_codec_path,
+					     XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
 					     _("path to RealPlayer codecs"),
 					     _("If you have RealPlayer installed, specify the path "
 					       "to its codec directory here. You can easily find "
@@ -634,12 +660,12 @@ static uint32_t supported_types[] = { BUF_VIDEO_RV20,
                                       BUF_VIDEO_RV40,
                                       0 };
 
-static decoder_info_t dec_info_real = {
+static const decoder_info_t dec_info_real = {
   supported_types,     /* supported types */
   7                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 18, "real", XINE_VERSION_CODE, &dec_info_real, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
