@@ -126,7 +126,12 @@ typedef struct directfb_driver_s {
   GC                           gc;
   int                          depth;
   x11osd                      *xoverlay;
+  
+  void                        *user_data;
+  void                       (*lock_display) (void *user_data);
+  void                       (*unlock_display) (void *user_data);
 #endif
+
   int                          ovl_changed;
   
   /* screen size */
@@ -142,29 +147,61 @@ typedef struct directfb_driver_s {
 
 typedef struct {
   video_driver_class_t         driver_class;
+  int                          visual_type;
   xine_t                      *xine;
 } directfb_class_t;
 
 
 #define DEFAULT_COLORKEY  0x202040
 
+#define DIRECTFB_OPTIONS  "no-banner,"\
+                          "bg-color=00000000,"\
+                          "no-vt-switch,"\
+                          "no-vt-switching,"\
+                          "no-sighandler,"\
+                          "no-deinit-check,"\
+                          "disable-module=linux_input,"\
+                          "disable-module=keyboard"
+                         
+#define XDIRECTFB_OPTIONS "no-banner,"\
+                          "no-sighandler,"\
+                          "no-deinit-check"
+
 
 #ifndef MAX
 # define MAX( a, b ) (((a) > (b)) ? (a) : (b))
 #endif
 
-#define YCBCR_TO_RGB( y, cb, cr, r, g, b ) do {\
-  int _y, _cb, _cr, _r, _g, _b;\
-  _y  = ((y) - 16) * 76309;\
-  _cb = (cb) - 128;\
-  _cr = (cr) - 128;\
-  _r = (_y                + _cr * 104597 + 0x8000) >> 16;\
-  _g = (_y - _cb *  25675 - _cr *  53279 + 0x8000) >> 16;\
-  _b = (_y + _cb * 132201                + 0x8000) >> 16;\
-  (r) = (_r < 0) ? 0 : ((_r > 255) ? 255 : _r);\
-  (g) = (_g < 0) ? 0 : ((_g > 255) ? 255 : _g);\
-  (b) = (_b < 0) ? 0 : ((_b > 255) ? 255 : _b);\
-} while (0)
+#define YCBCR_TO_RGB( y, cb, cr, r, g, b ) \
+  do { \
+    int _y, _cb, _cr, _r, _g, _b; \
+    _y  = ((y) - 16) * 76309; \
+    _cb = (cb) - 128; \
+    _cr = (cr) - 128; \
+    _r = (_y                + _cr * 104597 + 0x8000) >> 16; \
+    _g = (_y - _cb *  25675 - _cr *  53279 + 0x8000) >> 16; \
+    _b = (_y + _cb * 132201                + 0x8000) >> 16; \
+    (r) = (_r < 0) ? 0 : ((_r > 255) ? 255 : _r); \
+    (g) = (_g < 0) ? 0 : ((_g > 255) ? 255 : _g); \
+    (b) = (_b < 0) ? 0 : ((_b > 255) ? 255 : _b); \
+  } while (0)
+
+#define LOCK_DISPLAY() \
+  do { \
+    if (this->lock_display) \
+      this->lock_display (this->user_data); \
+    else \
+      XLockDisplay (this->display); \
+  } while (0)
+  
+#define UNLOCK_DISPLAY() \
+  do { \
+    if (this->unlock_display) \
+      this->unlock_display (this->user_data); \
+    else \
+      XUnlockDisplay (this->display); \
+  } while (0)
+    
 
 /*** driver functions ***/
 
@@ -304,12 +341,13 @@ static uint32_t directfb_colorkey_to_pixel (directfb_driver_t *this) {
 #endif
 
 static void directfb_clean_output_area (directfb_driver_t *this) {
-  if (this->visual_type == XINE_VISUAL_TYPE_X11) {
+  if (this->visual_type == XINE_VISUAL_TYPE_X11 ||
+      this->visual_type == XINE_VISUAL_TYPE_X11_2) {
 #ifdef HAVE_X11
     if (this->config.options & DLOP_DST_COLORKEY) {
       int i;
       
-      XLockDisplay (this->display);    
+      LOCK_DISPLAY();
       
       XSetForeground (this->display, this->gc, BlackPixel(this->display, this->screen));
     
@@ -334,13 +372,15 @@ static void directfb_clean_output_area (directfb_driver_t *this) {
     
       XFlush (this->display);
 
-      XUnlockDisplay (this->display);
+      UNLOCK_DISPLAY();
     }
 #endif
   }
   else if (!(this->caps & DLCAPS_SCREEN_LOCATION)) {
-    DFBRectangle rect[4];
-    int          i; 
+    DFBRectangle           rect[4];
+    DFBSurfaceCapabilities caps;
+    int                    bufs = 1;
+    int                    i;
 
     for (i = 0; i < 4; i++) {
       rect[i].x = MAX(this->sc.border[i].x, 0);
@@ -348,9 +388,18 @@ static void directfb_clean_output_area (directfb_driver_t *this) {
       rect[i].w = MAX(this->sc.border[i].w, 0);
       rect[i].h = MAX(this->sc.border[i].h, 0);
     }
-      
+    
+    this->surface->GetCapabilities (this->surface, &caps);
+    if (caps & DSCAPS_DOUBLE)
+      bufs++;
+    if (caps & DSCAPS_TRIPLE)
+      bufs++;
+
     this->surface->SetColor (this->surface, 0x00, 0x00, 0x00, 0xff);
-    this->surface->FillRectangles (this->surface, &rect[0], 4);
+    for (i = 0; i < bufs; i++) {  
+      this->surface->FillRectangles (this->surface, &rect[0], 4);
+      this->surface->Flip (this->surface, NULL, 0);
+    }
   }
 }
 
@@ -363,9 +412,9 @@ static void directfb_overlay_begin (vo_driver_t *this_gen,
   if (this->ovl_changed) {
 #ifdef HAVE_X11
     if (this->xoverlay) {
-      XLockDisplay (this->display);
+      LOCK_DISPLAY();
       x11osd_clear (this->xoverlay); 
-      XUnlockDisplay (this->display);
+      UNLOCK_DISPLAY();
     }
 #endif
     if (this->spic_surface) {
@@ -452,7 +501,7 @@ static void directfb_subpicture_paint (directfb_driver_t *this,
           if (!colors[index].a) {
             YCBCR_TO_RGB (color.y, color.cb, color.cr,
                           colors[index].r, colors[index].g, colors[index].b);
-            colors[index].a = alpha * 17;
+            colors[index].a = alpha | (alpha << 4);
           }
             
           lprintf ("color change to %02x%02x%02x%02x.\n",
@@ -507,9 +556,9 @@ static void directfb_overlay_blend (vo_driver_t *this_gen,
       return;
 #ifdef HAVE_X11
     if (this->xoverlay) {
-      XLockDisplay (this->display);
+      LOCK_DISPLAY();
       x11osd_blend (this->xoverlay, overlay); 
-      XUnlockDisplay (this->display);
+      UNLOCK_DISPLAY();
     }
 #endif
     if (this->spic_surface) {
@@ -539,9 +588,9 @@ static void directfb_overlay_end (vo_driver_t *this_gen, vo_frame_t *frame_gen)
   if (this->ovl_changed) {
 #ifdef HAVE_X11
     if (this->xoverlay) {
-      XLockDisplay (this->display);
+      LOCK_DISPLAY();
       x11osd_expose (this->xoverlay);
-      XUnlockDisplay (this->display);
+      UNLOCK_DISPLAY();
     }
 #endif
     if (this->spic_surface) {
@@ -646,8 +695,12 @@ static void directfb_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen
     
     this->layer->TestConfiguration (this->layer, &this->config, &failed);
     this->config.flags &= ~failed;
-    this->layer->SetConfiguration (this->layer, &this->config);
-    this->layer->GetConfiguration (this->layer, &this->config);
+    if (this->config.flags) {
+      this->layer->SetConfiguration (this->layer, &this->config);
+      this->layer->GetConfiguration (this->layer, &this->config);
+      this->surface->Release (this->surface);
+      this->layer->GetSurface (this->layer, &this->surface);
+    }
 
     lprintf ("failed=0x%08x.\n", failed);
   }
@@ -1068,16 +1121,17 @@ static int directfb_gui_data_exchange (vo_driver_t *this_gen,
     case XINE_GUI_SEND_DRAWABLE_CHANGED:
       lprintf ("drawable changed.\n");
 #ifdef HAVE_X11
-      if (this->visual_type == XINE_VISUAL_TYPE_X11) {
+      if (this->visual_type == XINE_VISUAL_TYPE_X11 ||
+          this->visual_type == XINE_VISUAL_TYPE_X11_2) {
         this->drawable = (Drawable) data;
-        XLockDisplay (this->display);
+        LOCK_DISPLAY();
         XFreeGC (this->display, this->gc);
         this->gc = XCreateGC (this->display, this->drawable, 0, NULL);
         if (this->xoverlay) {
           x11osd_drawable_changed (this->xoverlay, this->drawable);
           this->ovl_changed = 1;
         }          
-        XUnlockDisplay (this->display);
+        UNLOCK_DISPLAY();
         this->sc.force_redraw = 1;
       }
 #endif 
@@ -1087,11 +1141,12 @@ static int directfb_gui_data_exchange (vo_driver_t *this_gen,
     case XINE_GUI_SEND_EXPOSE_EVENT:
       lprintf ("expose event.\n");
 #ifdef HAVE_X11
-      if (this->visual_type == XINE_VISUAL_TYPE_X11) {
+      if (this->visual_type == XINE_VISUAL_TYPE_X11 ||
+          this->visual_type == XINE_VISUAL_TYPE_X11_2) {
         if (this->xoverlay) {
-          XLockDisplay (this->display);
-	        x11osd_expose (this->xoverlay);
-	        XUnlockDisplay (this->display);
+          LOCK_DISPLAY();
+          x11osd_expose (this->xoverlay);
+          UNLOCK_DISPLAY();
 	       }
       }
 #endif
@@ -1128,12 +1183,13 @@ static void directfb_dispose (vo_driver_t *this_gen) {
     this->cur_frame->vo_frame.dispose (&this->cur_frame->vo_frame);
         
 #ifdef HAVE_X11
-  if (this->visual_type == XINE_VISUAL_TYPE_X11) {
-    XLockDisplay (this->display);
+  if (this->visual_type == XINE_VISUAL_TYPE_X11 ||
+      this->visual_type == XINE_VISUAL_TYPE_X11_2) {
+    LOCK_DISPLAY();
     if (this->xoverlay)
       x11osd_destroy (this->xoverlay);
     XFreeGC (this->display, this->gc);
-    XUnlockDisplay (this->display);
+    UNLOCK_DISPLAY();
   }
 #endif
 
@@ -1171,6 +1227,8 @@ static void directfb_dispose (vo_driver_t *this_gen) {
 static void update_config_cb (void *data, xine_cfg_entry_t *entry) {
   directfb_driver_t *this = (directfb_driver_t *) data;
   
+  lprintf ("update_config_cb(%s).\n", entry->key);
+  
   if (strcmp (entry->key, "video.device.directfb_buffermode") == 0) {
     DFBDisplayLayerConfig config = { .flags = DLCONF_BUFFERMODE };
     
@@ -1186,8 +1244,8 @@ static void update_config_cb (void *data, xine_cfg_entry_t *entry) {
         break;
     }
     
-    if (config.buffermode != this->config.buffermode) {      
-      if (this->layer->SetConfiguration (this->layer, &config) != DFB_OK) {
+    if (config.buffermode != this->config.buffermode) {
+      if (this->layer->SetConfiguration (this->layer, &config ) != DFB_OK) {
         xprintf (this->xine, XINE_VERBOSITY_LOG,
                  "video_out_directfb: failed to set buffermode to %d!\n", 
                  entry->num_value);
@@ -1205,7 +1263,7 @@ static void update_config_cb (void *data, xine_cfg_entry_t *entry) {
     else
       config.options = this->config.options & ~DLOP_DST_COLORKEY;
     
-    if (config.options != this->config.options) {  
+    if (config.options != this->config.options) {
       if (this->layer->SetConfiguration (this->layer, &config) != DFB_OK) {
           xprintf (this->xine, XINE_VERBOSITY_LOG,
                    "video_out_directfb: failed to set colorkeying to %d!\n", 
@@ -1221,7 +1279,7 @@ static void update_config_cb (void *data, xine_cfg_entry_t *entry) {
     this->colorkey = entry->num_value;
     this->layer->SetDstColorKey (this->layer, (this->colorkey & 0xff0000) >> 16,
                                               (this->colorkey & 0x00ff00) >>  8,
-                                              (this->colorkey & 0x0000ff) >>  0);
+                                              (this->colorkey & 0x0000ff) >>  0); 
 #ifdef HAVE_X11
     if (this->xoverlay) {
       x11osd_colorkey (this->xoverlay, 
@@ -1386,7 +1444,14 @@ static void init_subpicture (directfb_driver_t *this) {
       config.flags       = DLCONF_PIXELFORMAT | DLCONF_OPTIONS;
       config.pixelformat = DSPF_ARGB;
       config.options     = DLOP_ALPHACHANNEL;
+      
       ret = this->underlay->SetConfiguration (this->underlay, &config);
+      if (ret) {
+        /* try AiRGB if the previous failed */
+        config.pixelformat = DSPF_AiRGB; 
+        ret = this->underlay->SetConfiguration (this->underlay, &config);
+      }
+      
       if (ret == DFB_OK) {
         this->underlay->AddRef (this->underlay);
         this->spic_layer = this->underlay;
@@ -1395,7 +1460,7 @@ static void init_subpicture (directfb_driver_t *this) {
       }
     }
   }
-  
+#if 0  
   /* most common type of supicture layer */
   if (!this->spic_layer) {
     IDirectFBScreen   *screen;
@@ -1434,7 +1499,7 @@ static void init_subpicture (directfb_driver_t *this) {
       }
     }
   }
-          
+#endif          
   if (this->spic_layer) {
     ret = this->spic_layer->GetSurface (this->spic_layer, &this->spic_surface);
     if (ret) {
@@ -1680,6 +1745,9 @@ static void directfb_frame_output_cb (void *user_data, int video_width, int vide
 static vo_driver_t *open_plugin_fb (video_driver_class_t *class_gen, const void *visual_gen) {
   directfb_class_t  *class  = (directfb_class_t *) class_gen;
   directfb_driver_t *this;
+  char              *args[] = { "xine", "--dfb:" DIRECTFB_OPTIONS };
+  int                argn   = 2;
+  char             **argp   = (char **) args;
   fb_visual_t       *visual = (fb_visual_t *) visual_gen;
   config_values_t   *config = class->xine->config;
   DFBDisplayLayerID  id;
@@ -1693,21 +1761,13 @@ static vo_driver_t *open_plugin_fb (video_driver_class_t *class_gen, const void
   this->xine        = class->xine;
   
   /* initialize DirectFB */ 
-  ret = DirectFBInit (NULL, NULL);
+  ret = DirectFBInit (&argn, &argp);
   if (ret != DFB_OK) {
     DirectFBError ("DirectFBInit()", ret);
     free (this);
     return NULL;
   }
   
-  DirectFBSetOption ("bg-color", "00000000" );
-  DirectFBSetOption ("no-vt-switch", NULL);
-  DirectFBSetOption ("no-vt-switching", NULL);
-  DirectFBSetOption ("no-sighandler", NULL );
-  DirectFBSetOption ("no-deinit-check", NULL );
-  DirectFBSetOption ("disable-module", "linux_input");
-  DirectFBSetOption ("disable-module", "keyboard");
-  
   /* create the main interface or retrieve an already existing one */
   ret = DirectFBCreate (&this->dfb);
   if (ret != DFB_OK) {
@@ -1869,6 +1929,9 @@ static const vo_info_t vo_info_directfb_fb = {
 static vo_driver_t *open_plugin_x11 (video_driver_class_t *class_gen, const void *visual_gen) {
   directfb_class_t  *class  = (directfb_class_t *) class_gen;
   directfb_driver_t *this;
+  char              *args[] = { "xine", "--dfb:" XDIRECTFB_OPTIONS };
+  int                argn   = 2;
+  char             **argp   = (char **) args;
   x11_visual_t      *visual = (x11_visual_t *) visual_gen;
   XWindowAttributes  attrs;
   IDirectFBScreen   *screen;
@@ -1879,20 +1942,17 @@ static vo_driver_t *open_plugin_x11 (video_driver_class_t *class_gen, const void
   if (!this)
     return NULL;
   
-  this->visual_type = XINE_VISUAL_TYPE_X11;
+  this->visual_type = class->visual_type;
   this->xine        = class->xine;
   
   /* initialize DirectFB */ 
-  ret = DirectFBInit (NULL, NULL);
+  ret = DirectFBInit (&argn, &argp);
   if (ret != DFB_OK) {
     DirectFBError ("DirectFBInit()", ret);
     free (this);
     return NULL;
   }
   
-  DirectFBSetOption ("no-sighandler", NULL );
-  DirectFBSetOption ("no-deinit-check", NULL );
-  
   /* create the main interface or retrieve an already existing one */
   ret = DirectFBCreate (&this->dfb);
   if (ret != DFB_OK) {
@@ -1989,6 +2049,12 @@ static vo_driver_t *open_plugin_x11 (video_driver_class_t *class_gen, const void
   this->sc.frame_output_cb = visual->frame_output_cb;
   this->sc.user_data       = visual->user_data;
   
+  if (this->visual_type == XINE_VISUAL_TYPE_X11_2) {
+    this->user_data      = visual->user_data;
+    this->lock_display   = visual->lock_display;
+    this->unlock_display = visual->unlock_display;
+  }    
+  
   if (this->colorkeying) {
     this->xoverlay = x11osd_create (this->xine, this->display, this->screen,
                                     this->drawable, X11OSD_COLORKEY);
@@ -2039,7 +2105,7 @@ static void *init_class_x11 (xine_t *xine, void *visual_gen) {
   directfb_class_t *this;
   x11_visual_t     *visual = (x11_visual_t *) visual_gen;
   const char       *error;
-  
+
   /* check DirectFB version */
   error = DirectFBCheckVersion( DIRECTFB_MAJOR_VERSION,
                                 DIRECTFB_MINOR_VERSION,
@@ -2066,26 +2132,44 @@ static void *init_class_x11 (xine_t *xine, void *visual_gen) {
   this->driver_class.get_description = get_description_x11;
   this->driver_class.dispose         = dispose_class_x11;
 
-  this->xine = xine;
+  this->visual_type = XINE_VISUAL_TYPE_X11;
+  this->xine        = xine;
 
   return this;
 }
 
+static void *init_class_x11_2 (xine_t *xine, void *visual_gen) {
+  directfb_class_t *this;
+
+  this = init_class_x11( xine, visual_gen );
+  if (this)
+    this->visual_type = XINE_VISUAL_TYPE_X11_2;
+    
+  return this;
+}
+
 static const vo_info_t vo_info_directfb_x11 = {
   8,                    /* priority    */
   XINE_VISUAL_TYPE_X11  /* visual type */
 };
+
+static const vo_info_t vo_info_directfb_x11_2 = {
+  8,                      /* priority    */
+  XINE_VISUAL_TYPE_X11_2  /* visual type */
+};
 #endif /* HAVE_X11 */
 
 /*********/
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, VIDEO_OUT_DRIVER_IFACE_VERSION, "DirectFB",
     XINE_VERSION_CODE, &vo_info_directfb_fb, init_class_fb },
 #ifdef HAVE_X11
   { PLUGIN_VIDEO_OUT, VIDEO_OUT_DRIVER_IFACE_VERSION, "XDirectFB",
     XINE_VERSION_CODE, &vo_info_directfb_x11, init_class_x11 },
+  { PLUGIN_VIDEO_OUT, VIDEO_OUT_DRIVER_IFACE_VERSION, "XDirectFB",
+    XINE_VERSION_CODE, &vo_info_directfb_x11_2, init_class_x11_2 },
 #endif
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
