@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_decoder.c,v 1.136 2005/10/30 02:18:35 miguelfreitas Exp $
+ * $Id: audio_decoder.c,v 1.138 2006/09/08 21:11:29 miguelfreitas Exp $
  *
  *
  * functions that implement audio decoding
@@ -226,6 +226,19 @@ static void *audio_decoder_loop (void *stream_gen) {
       }
       break;
 
+    case BUF_CONTROL_RESET_TRACK_MAP:
+      if (stream->audio_track_map_entries)
+      {
+        xine_event_t ui_event;
+
+        stream->audio_track_map_entries = 0;
+
+        ui_event.type        = XINE_EVENT_UI_CHANNELS_CHANGED;
+        ui_event.data_length = 0;
+        xine_event_send(stream, &ui_event);
+      }
+      break;
+
 
     default:
 
@@ -446,7 +459,7 @@ static void *audio_decoder_loop (void *stream_gen) {
   return NULL;
 }
 
-void _x_audio_decoder_init (xine_stream_t *stream) {
+int _x_audio_decoder_init (xine_stream_t *stream) {
 
   pthread_attr_t       pth_attrs;
   struct sched_param   pth_params;
@@ -454,7 +467,7 @@ void _x_audio_decoder_init (xine_stream_t *stream) {
 
   if (stream->audio_out == NULL) {
     stream->audio_fifo = _x_dummy_fifo_buffer_new (5, 8192);
-    return;
+    return 1;
   } else {
     int num_buffers;
     
@@ -497,11 +510,14 @@ void _x_audio_decoder_init (xine_stream_t *stream) {
                                &pth_attrs, audio_decoder_loop, stream)) != 0) {
       xprintf (stream->xine, XINE_VERBOSITY_DEBUG, 
 	       "audio_decoder: can't create new thread (%s)\n", strerror(err));
-      _x_abort();
+      stream->audio_thread_created = 0;
+      pthread_attr_destroy(&pth_attrs);
+      return 0;
     }
   
     pthread_attr_destroy(&pth_attrs);
   }
+  return 1;
 }
 
 void _x_audio_decoder_shutdown (xine_stream_t *stream) {
