@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer_types.c,v 1.101 2005/07/17 20:18:37 jstembridge Exp $
+ * $Id: buffer_types.c,v 1.107 2006/12/26 03:18:56 dgp85 Exp $
  *
  *
  * contents:
@@ -171,6 +171,7 @@ static video_db_t video_db[] = {
     meFOURCC('A', 'P', '4', '1'),
     meFOURCC('M', 'P', 'G', '3'),
     meFOURCC('C', 'O', 'L', '1'),
+    meFOURCC('3', 'I', 'V', 'D'),
     0
   },
   BUF_VIDEO_MSMPEG4_V3,
@@ -284,7 +285,8 @@ static video_db_t video_db[] = {
     else if (!strncasecmp (video, "yuv2", 4))
     this->video_type = BUF_VIDEO_YUY2;
     */
-    meFOURCC('y','u','v','2'),
+    meFOURCC('y', 'u', 'v', '2'),
+    meFOURCC('Y', 'U', 'Y', '2'),
     0
   },
   BUF_VIDEO_YUY2,
@@ -729,6 +731,38 @@ static video_db_t video_db[] = {
   BUF_VIDEO_FPS1,
   "Fraps FPS1"
 },
+{
+  {
+    meFOURCC('T','M','2','0'),
+    0
+  },
+  BUF_VIDEO_DUCKTM2,
+  "Duck TrueMotion 2"
+},
+{
+  {
+    meFOURCC('C','S','C','D'),
+    0
+  },
+  BUF_VIDEO_CSCD,
+  "CamStudio"
+},
+{
+  {
+    meFOURCC('Z','M','B','V'),
+    0
+  },
+  BUF_VIDEO_ZMBV,
+  "Zip Motion Blocks Video"
+},
+{
+  {
+    meFOURCC('K','M','V','C'),
+    0
+  },
+  BUF_VIDEO_KMVC,
+  "Karl Morton's Video Codec"
+},
 { { 0 }, 0, "last entry" }
 };
 
@@ -1066,6 +1100,13 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_ALAC,
   "Apple Lossless Audio Codec"
 },
+{
+  {
+    0x22,
+  },
+  BUF_AUDIO_TRUESPEECH,
+  "Truespeech"
+},
 {
   {
     0
@@ -1073,6 +1114,13 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_MPC,
   "Musepack"
 },
+{
+  {
+    0
+  },
+  BUF_AUDIO_WAVPACK,
+  "Wavpack"
+},
 { { 0 }, 0, "last entry" }
 };
 
