@@ -20,7 +20,7 @@
  * Demuxer helper functions
  * hide some xine engine details from demuxers and reduce code duplication
  *
- * $Id: demux.c,v 1.60 2005/10/30 02:18:35 miguelfreitas Exp $ 
+ * $Id: demux.c,v 1.66 2007/01/18 23:28:46 dgp85 Exp $ 
  */
 
 
@@ -85,6 +85,8 @@ void _x_demux_flush_engine (xine_stream_t *stream) {
   stream->video_fifo->clear(stream->video_fifo);
   stream->audio_fifo->clear(stream->audio_fifo);
   
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_RESET_DECODER;
   stream->video_fifo->put (stream->video_fifo, buf);
@@ -93,6 +95,8 @@ void _x_demux_flush_engine (xine_stream_t *stream) {
   buf->type = BUF_CONTROL_RESET_DECODER;
   stream->audio_fifo->put (stream->audio_fifo, buf);
   
+  pthread_mutex_unlock(&stream->demux_mutex);  
+
   /* on seeking we must wait decoder fifos to process before doing flush. 
    * otherwise we flush too early (before the old data has left decoders)
    */
@@ -124,6 +128,8 @@ void _x_demux_control_newpts( xine_stream_t *stream, int64_t pts, uint32_t flags
 
   buf_element_t *buf;
       
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_NEWPTS;
   buf->decoder_flags = flags;
@@ -135,6 +141,8 @@ void _x_demux_control_newpts( xine_stream_t *stream, int64_t pts, uint32_t flags
   buf->decoder_flags = flags;
   buf->disc_off = pts;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 /* sync with decoder fifos, making sure everything gets processed */
@@ -165,12 +173,16 @@ void _x_demux_control_headers_done (xine_stream_t *stream) {
     header_count_audio = 0;
   }
   
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf_video->type = BUF_CONTROL_HEADERS_DONE;
   stream->video_fifo->put (stream->video_fifo, buf_video);
 
   buf_audio->type = BUF_CONTROL_HEADERS_DONE;
   stream->audio_fifo->put (stream->audio_fifo, buf_audio);
 
+  pthread_mutex_unlock(&stream->demux_mutex);  
+
   while ((stream->header_count_audio < header_count_audio) || 
          (stream->header_count_video < header_count_video)) {
     struct timeval tv;
@@ -198,6 +210,8 @@ void _x_demux_control_start( xine_stream_t *stream ) {
 
   buf_element_t *buf;
 
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_START;
   stream->video_fifo->put (stream->video_fifo, buf);
@@ -205,12 +219,16 @@ void _x_demux_control_start( xine_stream_t *stream ) {
   buf = stream->audio_fifo->buffer_pool_alloc (stream->audio_fifo);
   buf->type = BUF_CONTROL_START;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 void _x_demux_control_end( xine_stream_t *stream, uint32_t flags ) {
 
   buf_element_t *buf;
 
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_END;
   buf->decoder_flags = flags;
@@ -220,12 +238,16 @@ void _x_demux_control_end( xine_stream_t *stream, uint32_t flags ) {
   buf->type = BUF_CONTROL_END;
   buf->decoder_flags = flags;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 void _x_demux_control_nop( xine_stream_t *stream, uint32_t flags ) {
 
   buf_element_t *buf;
 
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_NOP;
   buf->decoder_flags = flags;
@@ -235,6 +257,8 @@ void _x_demux_control_nop( xine_stream_t *stream, uint32_t flags ) {
   buf->type = BUF_CONTROL_NOP;
   buf->decoder_flags = flags;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 static void *demux_loop (void *stream_gen) {
@@ -289,6 +313,17 @@ static void *demux_loop (void *stream_gen) {
       status = stream->demux_plugin->get_status(stream->demux_plugin);
     }
 
+    /* delay sending finished event - used for image presentations */
+    while(stream->demux_thread_running &&
+          status == DEMUX_FINISHED && stream->delay_finish_event != 0){
+      pthread_mutex_unlock( &stream->demux_lock );
+      xine_usec_sleep(100000);
+      if( stream->delay_finish_event > 0 )
+        stream->delay_finish_event--;
+      pthread_mutex_lock( &stream->demux_lock );
+      status = stream->demux_plugin->get_status(stream->demux_plugin);
+    }
+
   } while( status == DEMUX_OK && stream->demux_thread_running &&
            !stream->emergency_brake);
 
@@ -301,7 +336,7 @@ static void *demux_loop (void *stream_gen) {
     finished_count_video = stream->finished_count_video + 1;
   pthread_mutex_unlock (&stream->counter_lock);
 
-  /* demux_thread_running is zero if demux loop has being stopped by user */
+  /* demux_thread_running is zero if demux loop has been stopped by user */
   non_user = stream->demux_thread_running;
   stream->demux_thread_running = 0;
   
@@ -413,7 +448,7 @@ int _x_demux_read_header( input_plugin_t *input, unsigned char *buffer, off_t si
   return read_size;
 }
 
-int _x_demux_check_extension (char *mrl, char *extensions){
+int _x_demux_check_extension (const char *mrl, const char *extensions){
   char *last_dot, *e, *ext_copy, *ext_work;
 
   ext_copy = strdup(extensions);
@@ -525,7 +560,8 @@ void _x_demux_send_data(fifo_buffer_t *fifo, uint8_t *data, int size,
 
   decoder_flags |= BUF_FLAG_FRAME_START;
 
-  while (fifo && size) {
+  _x_assert(size > 0);
+  while (fifo && size > 0) {
 
     buf = fifo->buffer_pool_alloc (fifo);
 
@@ -570,7 +606,8 @@ int _x_demux_read_send_data(fifo_buffer_t *fifo, input_plugin_t *input,
 
   decoder_flags |= BUF_FLAG_FRAME_START;
 
-  while (fifo && size) {
+  _x_assert(size > 0);
+  while (fifo && size > 0) {
 
     buf = fifo->buffer_pool_alloc (fifo);
 
@@ -632,8 +669,8 @@ void _x_demux_send_mrl_reference (xine_stream_t *stream, int alternative,
   data.e->alternative = alternative;
   data.e->start_time = start_time;
   data.e->duration = duration;
-  strcpy (data.e->mrl, mrl);
-  strcpy (data.e->mrl + mrl_len + 1, title ? title : "");
+  strcpy((char *)data.e->mrl, mrl);
+  strcpy((char *)data.e->mrl + mrl_len + 1, title ? title : "");
 
   event.type = XINE_EVENT_MRL_REFERENCE_EXT;
   xine_event_send (stream, &event);
