@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_decoder.c,v 1.160 2005/10/30 02:18:35 miguelfreitas Exp $
+ * $Id: video_decoder.c,v 1.162 2006/12/03 19:23:16 miguelfreitas Exp $
  *
  */
 
@@ -193,8 +193,13 @@ static void *video_decoder_loop (void *stream_gen) {
 
       /*
        * wait the output fifos to run dry before sending the notification event
-       * to the frontend. this test is only valid if there is only a single
-       * stream attached to the current output port.
+       * to the frontend. exceptions:
+       * 1) don't wait if there is more than one stream attached to the current
+       *    output port (the other stream might be sending data so we would be
+       *    here forever)
+       * 2) early_finish_event: send notification asap to allow gapless switch
+       * 3) slave stream: don't wait. get into an unblocked state asap to allow
+       *    new master actions.
        */
       while(1) {
         int num_bufs, num_streams;
@@ -204,7 +209,8 @@ static void *video_decoder_loop (void *stream_gen) {
         num_streams = stream->video_out->get_property(stream->video_out, VO_PROP_NUM_STREAMS);
         running_ticket->release(running_ticket, 0);
         
-        if( num_bufs > 0 && num_streams == 1 && !stream->early_finish_event )
+        if( num_bufs > 0 && num_streams == 1 && !stream->early_finish_event &&
+            stream->master == stream )
           xine_usec_sleep (10000);
         else
           break;
@@ -464,12 +470,12 @@ static void *video_decoder_loop (void *stream_gen) {
   return NULL;
 }
 
-void _x_video_decoder_init (xine_stream_t *stream) {
+int _x_video_decoder_init (xine_stream_t *stream) {
   
   if (stream->video_out == NULL) {
     stream->video_fifo = _x_dummy_fifo_buffer_new (5, 8192);
     stream->spu_track_map_entries = 0;
-    return;
+    return 1;
   } else {
      
     pthread_attr_t       pth_attrs;
@@ -494,6 +500,11 @@ void _x_video_decoder_init (xine_stream_t *stream) {
                                                       20, NULL, NULL);
   
     stream->video_fifo = _x_fifo_buffer_new (num_buffers, 8192);
+    if (stream->video_fifo == NULL) {
+      xine_log(stream->xine, XINE_LOG_MSG, "video_decoder: can't allocated video fifo\n");
+      return 0;
+    }
+    
     stream->spu_track_map_entries = 0;
   
     pthread_attr_init(&pth_attrs);
@@ -505,13 +516,16 @@ void _x_video_decoder_init (xine_stream_t *stream) {
     stream->video_thread_created = 1;
     if ((err = pthread_create (&stream->video_thread,
                                &pth_attrs, video_decoder_loop, stream)) != 0) {
-      fprintf (stderr, "video_decoder: can't create new thread (%s)\n",
-               strerror(err));
-      _x_abort();
+      xine_log (stream->xine, XINE_LOG_MSG, "video_decoder: can't create new thread (%s)\n",
+                strerror(err));
+      stream->video_thread_created = 0;
+      pthread_attr_destroy(&pth_attrs);
+      return 0;
     }
     
     pthread_attr_destroy(&pth_attrs);
   }
+  return 1;
 }
 
 void _x_video_decoder_shutdown (xine_stream_t *stream) {
