@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_interface.c,v 1.90 2005/07/03 20:31:29 miguelfreitas Exp $
+ * $Id: xine_interface.c,v 1.100 2006/12/19 19:10:52 dsalt Exp $
  *
  * convenience/abstraction layer, functions to implement
  * libxine's public interface
@@ -100,7 +100,23 @@ const char* xine_config_register_string (xine_t *self,
 					cb_data);
 
 }
+
+const char* xine_config_register_filename (xine_t *self,
+					   const char *key,
+					   const char *def_value,
+					   int req_type,
+					   const char *description,
+					   const char *help,
+					   int   exp_level,
+					   xine_config_cb_t changed_cb,
+					   void *cb_data) {
   
+  return self->config->register_filename (self->config,
+					  key, def_value, req_type,
+					  description, help, exp_level,
+					  changed_cb, cb_data);
+}
+
 int xine_config_register_range (xine_t *self,
 				const char *key,
 				int def_value,
@@ -326,6 +342,11 @@ int xine_port_send_gui_data (xine_video_port_t *vo,
 }
 
 void xine_set_param (xine_stream_t *stream, int param, int value) {
+  /* Avoid crashing */
+  if ( ! stream ) {
+    lprintf ("xine_interface: xine_set_param called with NULL stream.\n");
+    return;
+  }
 
   switch (param) {
   case XINE_PARAM_SPEED:
@@ -474,6 +495,18 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
       stream->broadcaster = NULL;
     }
     break;
+  
+  case XINE_PARAM_EARLY_FINISHED_EVENT:
+    stream->early_finish_event = value;
+    break;
+  
+  case XINE_PARAM_DELAY_FINISHED_EVENT:
+    stream->delay_finish_event = value;
+    break;
+  
+  case XINE_PARAM_GAPLESS_SWITCH:
+    stream->gapless_switch = value;
+    break;
     
   default:
     xprintf (stream->xine, XINE_VERBOSITY_DEBUG,
@@ -590,6 +623,8 @@ int xine_get_param (xine_stream_t *stream, int param) {
   case XINE_PARAM_VO_ZOOM_X:
   case XINE_PARAM_VO_ZOOM_Y:
   case XINE_PARAM_VO_TVMODE:
+  case XINE_PARAM_VO_WINDOW_WIDTH:
+  case XINE_PARAM_VO_WINDOW_HEIGHT:
   case XINE_PARAM_VO_CROP_LEFT:
   case XINE_PARAM_VO_CROP_RIGHT:
   case XINE_PARAM_VO_CROP_TOP:
@@ -621,6 +656,18 @@ int xine_get_param (xine_stream_t *stream, int param) {
     else
       ret = 0;
     break;
+  
+  case XINE_PARAM_EARLY_FINISHED_EVENT:
+    ret = stream->early_finish_event;
+    break;
+  
+  case XINE_PARAM_DELAY_FINISHED_EVENT:
+    ret = stream->delay_finish_event;
+    break;
+  
+  case XINE_PARAM_GAPLESS_SWITCH:
+    ret = stream->gapless_switch;
+    break;
 
   default:
     xprintf (stream->xine, XINE_VERBOSITY_DEBUG,
@@ -797,26 +844,28 @@ const char *const *xine_post_list_outputs(xine_post_t *this_gen) {
 
 xine_post_in_t *xine_post_input(xine_post_t *this_gen, const char *name) {
   post_plugin_t  *this = (post_plugin_t *)this_gen;
-  xine_post_in_t *input;
+  xine_list_iterator_t ite;
   
-  input = xine_list_first_content(this->input);
-  while (input) {
+  ite = xine_list_front(this->input);
+  while (ite) {
+    xine_post_in_t *input = xine_list_get_value(this->input, ite);
     if (strcmp(input->name, name) == 0)
       return input;
-    input = xine_list_next_content(this->input);
+    ite = xine_list_next(this->input, ite);
   }
   return NULL;
 }
 
 xine_post_out_t *xine_post_output(xine_post_t *this_gen, const char *name) {
   post_plugin_t   *this = (post_plugin_t *)this_gen;
-  xine_post_out_t *output;
+  xine_list_iterator_t ite;
   
-  output = xine_list_first_content(this->output);
-  while (output) {
+  ite = xine_list_front(this->output);
+  while (ite) {
+    xine_post_out_t *output = xine_list_get_value(this->output, ite);
     if (strcmp(output->name, name) == 0)
       return output;
-    output = xine_list_next_content(this->output);
+    ite = xine_list_next(this->output, ite);
   }
   return NULL;
 }
@@ -879,28 +928,29 @@ int _x_message(xine_stream_t *stream, int type, ...) {
   int                     n;
   va_list                 ap;
   char                   *s, *params;
-  char                   *args[1024];
+  char                   *args[1025];
   static char            *std_explanation[] = {
     "",
-    "Warning:",
-    "Unknown host:",
-    "Unknown device:",
-    "Network unreachable",
-    "Connection refused:",
-    "File not found:",
-    "Read error from:",
-    "Error loading library:",
-    "Encrypted media stream detected",
-    "Security message:",
-    "Audio device unavailable",
-    "Permission error",
+    N_("Warning:"),
+    N_("Unknown host:"),
+    N_("Unknown device:"),
+    N_("Network unreachable"),
+    N_("Connection refused:"),
+    N_("File not found:"),
+    N_("Read error from:"),
+    N_("Error loading library:"),
+    N_("Encrypted media stream detected"),
+    N_("Security message:"),
+    N_("Audio device unavailable"),
+    N_("Permission error"),
+    N_("File is empty:"),
   };
 
   if (!stream) return 0;
 
-  if( type >= 0 && type < sizeof(std_explanation)/
+  if( type >= 0 && (size_t)type < sizeof(std_explanation)/
                            sizeof(std_explanation[0]) ) {
-    explanation = std_explanation[type];
+    explanation = _(std_explanation[type]);
     size = strlen(explanation)+1;
   } else {
     explanation = NULL;
