@@ -1,18 +1,18 @@
-/* 
- * Copyright (C) 2001 the xine project
- * 
- * This file is part of xine, a unix video player.
- * 
+/*
+ * Copyright (C) 2001-2004 the xine project
+ *
+ * This file is part of xine, a free video player.
+ *
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * xine is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
@@ -25,7 +25,7 @@
  * Miguel Freitas split the #ifdefs into several specialized functions that
  * are benchmarked at runtime by xine. Some original comments from Nick
  * have been preserved documenting some MMX/SSE oddities.
- * Also added kernel memcpy function that seems faster than glibc one.
+ * Also added kernel memcpy function that seems faster than libc one.
  *
  */
 
@@ -33,18 +33,27 @@
 #include "config.h"
 #endif
 
-#ifdef ARCH_PPC
+#if defined (ARCH_PPC) && !defined (HOST_OS_DARWIN)
 #include "ppcasm_string.h"
 #endif
+
+#ifdef HAVE_SYS_TIMES_H
 #include <sys/times.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
+
+#define LOG_MODULE "memcpy"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
-#include "xineutils.h"
- 
+
 void *(* xine_fast_memcpy)(void *to, const void *from, size_t len);
- 
+
 /* Original comments from mplayer (file: aclib.c)
  This part of code was taken by me from Linux-2.4.3 and slightly modified
 for MMX, MMX2, SSE instruction set. I have done it since linux uses page aligned
@@ -93,7 +102,7 @@ If you have questions please contact with me: Nick Kurshev: nickols_k@mail.ru.
     of memory copying on PentMMX, Celeron-1 and P2 upto 12% versus
     standard (non MMX-optimized) version.
     Note: on K6-2+ it speedups memory copying upto 25% and
-          on K7 and P3 about 500% (5 times). 
+          on K7 and P3 about 500% (5 times).
 */
 
 /* Additional notes on gcc assembly and processors: [MF]
@@ -102,15 +111,16 @@ prefetch0, prefetch1, prefetch2 which are not recognized by my gcc.
 prefetchnta is supported both on athlon and pentium 3.
 
 therefore i will take off prefetchnta instructions from the mmx1 version
-to avoid problems on pentium mmx and k6-2.  
+to avoid problems on pentium mmx and k6-2.
 
 quote of the day:
 "Using prefetches efficiently is more of an art than a science"
 */
 
 
-#ifdef ARCH_X86
+#if defined(ARCH_X86) || defined(ARCH_X86_64)
 
+#ifndef _MSC_VER
 /* for small memory blocks (<256 bytes) this version is faster */
 #define small_memcpy(to,from,n)\
 {\
@@ -123,7 +133,10 @@ __asm__ __volatile__(\
 }
 
 /* linux kernel __memcpy (from: /include/asm/string.h) */
-static inline void * __memcpy(void * to, const void * from, size_t n)
+static __inline__ void * linux_kernel_memcpy_impl (
+			       void * to,
+			       const void * from,
+			       size_t n)
 {
 int d0, d1, d2;
 
@@ -143,7 +156,7 @@ int d0, d1, d2;
     : "=&c" (d0), "=&D" (d1), "=&S" (d2)
     :"0" (n/4), "q" (n),"1" ((long) to),"2" ((long) from)
     : "memory");
-  
+
   return (to);
 }
 
@@ -161,16 +174,21 @@ static void * sse_memcpy(void * to, const void * from, size_t len)
   void *retval;
   size_t i;
   retval = to;
-    
+
   /* PREFETCH has effect even for MOVSB instruction ;) */
   __asm__ __volatile__ (
     "   prefetchnta (%0)\n"
+    "   prefetchnta 32(%0)\n"
     "   prefetchnta 64(%0)\n"
+    "   prefetchnta 96(%0)\n"
     "   prefetchnta 128(%0)\n"
+    "   prefetchnta 160(%0)\n"
     "   prefetchnta 192(%0)\n"
+    "   prefetchnta 224(%0)\n"
     "   prefetchnta 256(%0)\n"
+    "   prefetchnta 288(%0)\n"
     : : "r" (from) );
-    
+
   if(len >= MIN_LEN)
   {
     register unsigned long int delta;
@@ -190,6 +208,7 @@ static void * sse_memcpy(void * to, const void * from, size_t len)
       {
         __asm__ __volatile__ (
         "prefetchnta 320(%0)\n"
+       "prefetchnta 352(%0)\n"
         "movups (%0), %%xmm0\n"
         "movups 16(%0), %%xmm1\n"
         "movups 32(%0), %%xmm2\n"
@@ -199,10 +218,10 @@ static void * sse_memcpy(void * to, const void * from, size_t len)
         "movntps %%xmm2, 32(%1)\n"
         "movntps %%xmm3, 48(%1)\n"
         :: "r" (from), "r" (to) : "memory");
-        ((const unsigned char *)from)+=64;
-        ((unsigned char *)to)+=64;
+        from = ((const unsigned char *)from) + 64;
+        to = ((unsigned char *)to) + 64;
       }
-    else 
+    else
       /*
          Only if SRC is aligned on 16-byte boundary.
          It allows to use movaps instead of movups, which required data
@@ -212,6 +231,7 @@ static void * sse_memcpy(void * to, const void * from, size_t len)
       {
         __asm__ __volatile__ (
         "prefetchnta 320(%0)\n"
+       "prefetchnta 352(%0)\n"
         "movaps (%0), %%xmm0\n"
         "movaps 16(%0), %%xmm1\n"
         "movaps 32(%0), %%xmm2\n"
@@ -221,19 +241,17 @@ static void * sse_memcpy(void * to, const void * from, size_t len)
         "movntps %%xmm2, 32(%1)\n"
         "movntps %%xmm3, 48(%1)\n"
         :: "r" (from), "r" (to) : "memory");
-        ((const unsigned char *)from)+=64;
-        ((unsigned char *)to)+=64;
+        from = ((const unsigned char *)from) + 64;
+        to = ((unsigned char *)to) + 64;
       }
     /* since movntq is weakly-ordered, a "sfence"
      * is needed to become ordered again. */
     __asm__ __volatile__ ("sfence":::"memory");
-    /* enables to use FPU */
-    __asm__ __volatile__ ("emms":::"memory");
   }
   /*
    *	Now do the tail of the block
    */
-  if(len) __memcpy(to, from, len);
+  if(len) linux_kernel_memcpy_impl(to, from, len);
   return retval;
 }
 
@@ -276,19 +294,19 @@ static void * mmx_memcpy(void * to, const void * from, size_t len)
       "movq %%mm6, 48(%1)\n"
       "movq %%mm7, 56(%1)\n"
       :: "r" (from), "r" (to) : "memory");
-      ((const unsigned char *)from)+=64;
-      ((unsigned char *)to)+=64;
+      from = ((const unsigned char *)from) + 64;
+      to = ((unsigned char *)to) + 64;
     }
     __asm__ __volatile__ ("emms":::"memory");
   }
   /*
    *	Now do the tail of the block
    */
-  if(len) __memcpy(to, from, len);
+  if(len) linux_kernel_memcpy_impl(to, from, len);
   return retval;
 }
 
-void * mmx2_memcpy(void * to, const void * from, size_t len)
+static void * mmx2_memcpy(void * to, const void * from, size_t len)
 {
   void *retval;
   size_t i;
@@ -297,10 +315,15 @@ void * mmx2_memcpy(void * to, const void * from, size_t len)
   /* PREFETCH has effect even for MOVSB instruction ;) */
   __asm__ __volatile__ (
     "   prefetchnta (%0)\n"
+    "   prefetchnta 32(%0)\n"
     "   prefetchnta 64(%0)\n"
+    "   prefetchnta 96(%0)\n"
     "   prefetchnta 128(%0)\n"
+    "   prefetchnta 160(%0)\n"
     "   prefetchnta 192(%0)\n"
+    "   prefetchnta 224(%0)\n"
     "   prefetchnta 256(%0)\n"
+    "   prefetchnta 288(%0)\n"
     : : "r" (from) );
 
   if(len >= MIN_LEN)
@@ -320,6 +343,7 @@ void * mmx2_memcpy(void * to, const void * from, size_t len)
     {
       __asm__ __volatile__ (
       "prefetchnta 320(%0)\n"
+      "prefetchnta 352(%0)\n"
       "movq (%0), %%mm0\n"
       "movq 8(%0), %%mm1\n"
       "movq 16(%0), %%mm2\n"
@@ -337,8 +361,8 @@ void * mmx2_memcpy(void * to, const void * from, size_t len)
       "movntq %%mm6, 48(%1)\n"
       "movntq %%mm7, 56(%1)\n"
       :: "r" (from), "r" (to) : "memory");
-      ((const unsigned char *)from)+=64;
-      ((unsigned char *)to)+=64;
+      from = ((const unsigned char *)from) + 64;
+      to = ((unsigned char *)to) + 64;
     }
      /* since movntq is weakly-ordered, a "sfence"
      * is needed to become ordered again. */
@@ -348,85 +372,101 @@ void * mmx2_memcpy(void * to, const void * from, size_t len)
   /*
    *	Now do the tail of the block
    */
-  if(len) __memcpy(to, from, len);
+  if(len) linux_kernel_memcpy_impl(to, from, len);
   return retval;
 }
 
 static void *linux_kernel_memcpy(void *to, const void *from, size_t len) {
-  return __memcpy(to,from,len);
+  return linux_kernel_memcpy_impl(to,from,len);
 }
-
+#endif /* _MSC_VER */
 #endif /* ARCH_X86 */
 
 static struct {
   char *name;
   void *(* function)(void *to, const void *from, size_t len);
-  unsigned long long time;
+
+  uint64_t time; /* This type could be used for non-MSC build too! */
+
   uint32_t cpu_require;
-} memcpy_method[] = 
+} memcpy_method[] =
 {
   { NULL, NULL, 0, 0 },
-  { "glibc memcpy()", memcpy, 0, 0 },
-#ifdef ARCH_X86
+  { "libc memcpy()", memcpy, 0, 0 },
+#if (defined(ARCH_X86) || defined(ARCH_X86_64)) && !defined(_MSC_VER)
   { "linux kernel memcpy()", linux_kernel_memcpy, 0, 0 },
   { "MMX optimized memcpy()", mmx_memcpy, 0, MM_MMX },
   { "MMXEXT optimized memcpy()", mmx2_memcpy, 0, MM_MMXEXT },
   { "SSE optimized memcpy()", sse_memcpy, 0, MM_MMXEXT|MM_SSE },
 #endif /* ARCH_X86 */
-#ifdef ARCH_PPC
+#if defined (ARCH_PPC) && !defined (HOST_OS_DARWIN)
   { "ppcasm_memcpy()", ppcasm_memcpy, 0, 0 },
-  { "ppcasm_cacheable_memcpy()", ppcasm_cacheable_memcpy, 0, 0 },
-#endif /* ARCH_PPC */
+  { "ppcasm_cacheable_memcpy()", ppcasm_cacheable_memcpy, 0, MM_ACCEL_PPC_CACHE32 },
+#endif /* ARCH_PPC && !HOST_OS_DARWIN */
   { NULL, NULL, 0, 0 }
 };
 
-#ifdef ARCH_X86
-static unsigned long long int rdtsc()
+#if (defined(ARCH_X86) || defined(ARCH_X86_64)) && defined(HAVE_SYS_TIMES_H)
+static int64_t rdtsc(int config_flags)
 {
-  unsigned long long int x;
-  __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));     
-  return x;
+  int64_t x;
+
+  /* that should prevent us from trying cpuid with old cpus */
+  if( config_flags & MM_MMX ) {
+    __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
+    return x;
+  } else {
+    return times(NULL);
+  }
 }
 #else
-static unsigned long long int rdtsc()
+
+static uint64_t rdtsc(int config_flags)
 {
   /* FIXME: implement an equivalent for using optimized memcpy on other
             architectures */
-#warning "Using rdtsc() hack (via times(NULL))"
-  return times(NULL);
+#ifdef HAVE_SYS_TIMES_H
+  struct tms tp;
+  return times(&tp);
+#else
+	return ((uint64_t)0);
+#endif /* HAVE_SYS_TIMES_H */
 }
 #endif
 
-static void update_fast_memcpy(void *this_gen, cfg_entry_t *entry)
-{
-  static int config_flags = -1;
-    
+static void update_fast_memcpy(void *user_data, xine_cfg_entry_t *entry) {
+  static int   config_flags = -1;
+  xine_t      *xine = (xine_t *) user_data;
+  int          method;
+
   config_flags = xine_mm_accel();
 
-  if( entry->num_value != 0 && 
-     (config_flags & memcpy_method[entry->num_value].cpu_require) == 
-      memcpy_method[entry->num_value].cpu_require ) {
-    printf("xine: using %s\n", memcpy_method[entry->num_value].name );
-    xine_fast_memcpy = memcpy_method[entry->num_value].function;
+  method = entry->num_value;
+
+  if (method != 0
+      && (config_flags & memcpy_method[method].cpu_require) ==
+      memcpy_method[method].cpu_require ) {
+    lprintf("using %s\n", memcpy_method[method].name );
+    xine_fast_memcpy = memcpy_method[method].function;
     return;
   } else {
-    printf("xine: will probe memcpy on startup\n" );
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "xine: will probe memcpy on startup\n" );
   }
 }
 
 #define BUFSIZE 1024*1024
-void xine_probe_fast_memcpy(config_values_t *config)
+void xine_probe_fast_memcpy(xine_t *xine)
 {
-  unsigned long long t;
-  char *buf1, *buf2;
-  int i, j, best;
-  int config_flags = -1;
-  static char *memcpy_methods[] = { 
-    "probe", "glibc",
-#ifdef ARCH_X86
-    "kernel", "mmx", "mmxext", "sse", 
+  uint64_t          t;
+  char             *buf1, *buf2;
+  int               i, j, best;
+  int               config_flags = -1;
+  static char      *memcpy_methods[] = {
+    "probe", "libc",
+#if (defined(ARCH_X86) || defined(ARCH_X86_64)) && !defined(_MSC_VER)
+    "kernel", "mmx", "mmxext", "sse",
 #endif
-#ifdef ARCH_PPC
+#if defined (ARCH_PPC) && !defined (HOST_OS_DARWIN)
     "ppcasm_memcpy", "ppcasm_cacheable_memcpy",
 #endif
     NULL
@@ -434,58 +474,63 @@ void xine_probe_fast_memcpy(config_values_t *config)
   
   config_flags = xine_mm_accel();
 
-  best = config->register_enum (config, "misc.memcpy_method", 0,
-				memcpy_methods, 
-				_("Memcopy method to use in xine for large data chunks."),
-				NULL, update_fast_memcpy, NULL);
-  
+  best = xine->config->register_enum (xine->config, "engine.performance.memcpy_method", 0,
+				      memcpy_methods,
+				      _("memcopy method used by xine"),
+				      _("The copying of large memory blocks is one of the most "
+					"expensive operations on todays computers. Therefore xine "
+					"provides various tuned methods to do this copying. "
+					"Usually, the best method is detected automatically."),
+				      20, update_fast_memcpy, (void *) xine);
+
   /* check if function is configured and valid for this machine */
-  if( best != 0 && 
-     (config_flags & memcpy_method[best].cpu_require) == 
+  if( best != 0 &&
+     (config_flags & memcpy_method[best].cpu_require) ==
       memcpy_method[best].cpu_require ) {
-    printf("xine: using %s\n", memcpy_method[best].name );
+    lprintf("using %s\n", memcpy_method[best].name );
     xine_fast_memcpy = memcpy_method[best].function;
     return;
   }
 
   best = 0;
-  
+
   xine_fast_memcpy = memcpy;
-  
+
   if( (buf1 = malloc(BUFSIZE)) == NULL )
     return;
-    
+
   if( (buf2 = malloc(BUFSIZE)) == NULL ) {
     free(buf1);
     return;
   }
 
-  printf("Benchmarking memcpy methods (smaller is better):\n");
+  xprintf(xine, XINE_VERBOSITY_LOG, _("Benchmarking memcpy methods (smaller is better):\n"));
   /* make sure buffers are present on physical memory */
-  memcpy(buf1,buf2,BUFSIZE);
+  memset(buf1,0,BUFSIZE);
+  memset(buf2,0,BUFSIZE);
 
   for(i=1; memcpy_method[i].name; i++)
   {
-    if( (config_flags & memcpy_method[i].cpu_require) != 
+    if( (config_flags & memcpy_method[i].cpu_require) !=
          memcpy_method[i].cpu_require )
       continue;
 
-    t = rdtsc();
-    for(j=0;j<50;j++) {	  
+    t = rdtsc(config_flags);
+    for(j=0;j<50;j++) {
       memcpy_method[i].function(buf2,buf1,BUFSIZE);
       memcpy_method[i].function(buf1,buf2,BUFSIZE);
-    }     
+    }
 
-    t = rdtsc() - t;
+    t = rdtsc(config_flags) - t;
     memcpy_method[i].time = t;
-    
-    printf("\t%s : %lld\n",memcpy_method[i].name, t);
-    
+
+    xprintf(xine, XINE_VERBOSITY_LOG, "\t%s : %" PRIu64 "\n", memcpy_method[i].name, t);
+
     if( best == 0 || t < memcpy_method[best].time )
       best = i;
   }
 
-  config->update_num (config, "misc.memcpy_method", best);
+  xine->config->update_num (xine->config, "engine.performance.memcpy_method", best);
 
   free(buf1);
   free(buf2);
